library IEEE;
use IEEE.std_logic_1164.all;
USE std.textio.all;
-- Xilinx simulation models
library unisim;
use unisim.vcomponents.all;
-- ddr_controller components
library work;
USE work.ddr_controller_comp.all;
-- DDR SDRAM behaviorial model
library work;
USE work.mt46v4m16_PKG.all;

entity test_E is
end test_E;

architecture test_ARCH of test_E is
  constant PERIOD 	: time := 10 ns;
  constant HALF_PERIOD 	: time := PERIOD/2;
  constant DELAY_TIME  	: time := PERIOD/10;

  signal sys_addr 	: std_logic_vector(SYS_ADDR_MSB downto 0);
  signal sys_cmd  	: std_logic_vector(7 downto 1);
  signal sys_reset_n  	: std_logic;
  signal sys_clk        : std_logic;
  signal sys_clk_fb     : std_logic;
  signal sys_ref_ack    : std_logic;  
  signal sys_data_valid : std_logic;  
  signal sys_data_i	: std_logic_vector(SYS_DATA_MSB downto 0);
  
  signal ddr_ad		: std_logic_vector(DDR_ADDR_MSB downto 0);
  signal ddr_dq		: std_logic_vector(DDR_DATA_MSB downto 0);
  signal ddr_dm		: std_logic_vector(1 downto 0);
  signal ddr_ba		: std_logic_vector(1 downto 0);
  signal ddr_dqs	: std_logic_vector(1 downto 0);
  signal ddr_rasb	: std_logic;
  signal ddr_casb	: std_logic;
  signal ddr_web	: std_logic;
  signal ddr_clk        : std_logic;
  signal ddr_clkb       : std_logic;
  signal ddr_csb        : std_logic;
  signal ddr_cke        : std_logic;
  signal sys_data_o	: std_logic_vector(SYS_DATA_MSB downto 0);
  

  type ddr_cmd is (LOAD_MR, AUTO_REFRESH, PRECHARGE, ACT, WRITEA, READA, BURST_STOP, NOP, UNKNOWN);
  signal ddr_controls : std_logic_vector(2 downto 0);
    
  component mt46v4m16
    generic(
      addr_bits     : INTEGER := 12;
      clk_start     : time    :=  5 ns;
      clk_period    : time    := 10 ns;
      continue_time : time    := 10 ns;
      data_bits     : INTEGER := 16
      );

    PORT (
      Dq    : INOUT STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0) := (OTHERS => 'Z');
      Dqs   : INOUT STD_LOGIC_VECTOR (1 DOWNTO 0) := "ZZ";
      Addr  : IN    STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0) := (OTHERS => '0');
      Ba    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0) := "00";
      Clk   : IN    STD_LOGIC := '0';
      Clk_n : IN    STD_LOGIC := '1';
      Cke   : IN    STD_LOGIC := '0';
      Cs_n  : IN    STD_LOGIC := '1';
      Ras_n : IN    STD_LOGIC := '0';
      Cas_n : IN    STD_LOGIC := '0';
      We_n  : IN    STD_LOGIC := '0';
      Dm    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0) := "00"
    );  
  END component;

begin
  uut: top
    port map (
      ddr_ad => ddr_ad, ddr_dm => ddr_dm, ddr_ba => ddr_ba, 
      ddr_rasb => ddr_rasb, ddr_casb => ddr_casb, ddr_web => ddr_web, 
      ddr_clk => ddr_clk, ddr_clkb => ddr_clkb, ddr_dqs => ddr_dqs, 
      ddr_csb => ddr_csb, ddr_cke => ddr_cke, ddr_dq => ddr_dq, 
      sys_addr => sys_addr, sys_data_i => sys_data_i, 
      sys_cmd => sys_cmd, sys_reset_n => sys_reset_n, sys_clk => sys_clk, 
      sys_clk_fb => sys_clk_fb, sys_data_o => sys_data_o, sys_ref_ack => sys_ref_ack,
      sys_data_valid => sys_data_valid
      );

  u_ddr1: mt46v4m16
    port map (
      Clk => ddr_clk, Clk_n => ddr_clkb, Cs_n => ddr_csb, 
      Cke => ddr_cke, Ba => ddr_ba, Addr => ddr_ad, 
      Ras_n => ddr_rasb, Cas_n => ddr_casb, We_n => ddr_web, 
      Dm => ddr_dm, Dq => ddr_dq(15 downto 0), Dqs => ddr_dqs
      );

  u_ddr2: mt46v4m16
    port map (
      Clk => ddr_clk, Clk_n => ddr_clkb, Cs_n => ddr_csb, 
      Cke => ddr_cke, Ba => ddr_ba, Addr => ddr_ad, 
      Ras_n => ddr_rasb, Cas_n => ddr_casb, We_n => ddr_web, 
      Dm => ddr_dm, Dq => ddr_dq(31 downto 16), Dqs => ddr_dqs
      );

  u_ddr3: mt46v4m16
    port map (
      Clk => ddr_clk, Clk_n => ddr_clkb, Cs_n => ddr_csb, 
      Cke => ddr_cke, Ba => ddr_ba, Addr => ddr_ad, 
      Ras_n => ddr_rasb, Cas_n => ddr_casb, We_n => ddr_web, 
      Dm => ddr_dm, Dq => ddr_dq(47 downto 32), Dqs => ddr_dqs
      );

  u_ddr4: mt46v4m16
    port map (
      Clk => ddr_clk, Clk_n => ddr_clkb, Cs_n => ddr_csb, 
      Cke => ddr_cke, Ba => ddr_ba, Addr => ddr_ad, 
      Ras_n => ddr_rasb, Cas_n => ddr_casb, We_n => ddr_web, 
      Dm => ddr_dm, Dq => ddr_dq(63 downto 48), Dqs => ddr_dqs
      );
 
  sys_clk_fb <= ddr_clk;

  CLKGEN: process (sys_clk) 
  begin
    if sys_clk = '1' then
      sys_clk <= '0' after HALF_PERIOD;
    else
      sys_clk <= '1' after HALF_PERIOD;
    end if;
  end process;

  STIMULUS: process
    procedure run_test (
      burst_length, cas_latency : in std_logic_vector (2 downto 0)) is
    begin  -- run_test
      report "run_test" severity note;
      wait for (PERIOD);     sys_cmd <= cmd_sys_nop; --NOP;
      wait for (PERIOD);     sys_cmd <= cmd_sys_load_mr; -- MRS;
      sys_addr <= "000000000000000" & cas_latency & '0' & burst_length;
      wait for (PERIOD);     sys_cmd <= cmd_sys_nop; --NOP;
      wait for (PERIOD);     sys_cmd <= cmd_sys_precharge; --PRECHARGE_ALL_BANKS;
      wait for (PERIOD);     sys_cmd <= cmd_sys_nop; --NOP;
      wait for (2*PERIOD);   sys_cmd <= cmd_sys_refresh; --AUTO_REFRESH;
      wait for (PERIOD);     sys_cmd <= cmd_sys_nop; --NOP;
      wait for (8*PERIOD);   sys_cmd <= cmd_sys_refresh; --AUTO_REFRESH;
      wait for (PERIOD);     sys_cmd <= cmd_sys_nop; --NOP;
      wait for (8*PERIOD);   sys_cmd <= cmd_sys_write;  --WRITE_AUTOPRECHARGE
                             --{ba,row_addr,col_addr} = {2'b01, 12'h200, 8'h20}
                             sys_addr <= "00" & x"200" & x"10";
                             sys_data_i <= x"1020";
                             wait for PERIOD;  
                             sys_data_i <= x"0304";
                             wait for PERIOD;  
                             sys_data_i <= x"5060";
                             wait for PERIOD;  
                             sys_data_i <= x"0708";
      wait for (PERIOD);     sys_cmd <= cmd_sys_nop; --NOP;
      wait for (10*PERIOD);  sys_cmd <= cmd_sys_read; --READ_AUTOPRECHARGE(2'b00, 12'h200, 8'h10);
                             --{ba,row_addr,col_addr} = {2'b01, 12'h200, 8'h20}
                             sys_addr <= "00" & x"200" & x"10";
      wait for (PERIOD);     sys_cmd <= cmd_sys_nop; --NOP;
      wait for (19*PERIOD);  sys_cmd <= cmd_sys_write;  --WRITE_AUTOPRECHARGE
                             sys_addr <= "01" & x"200" & x"20";
                             --{ba,row_addr,col_addr} = {2'b01, 12'h200, 8'h20}
                             sys_data_i <= x"0987";
                             wait for PERIOD;  
                             sys_data_i <= x"9876";
                             wait for PERIOD;  
                             sys_data_i <= x"7654";
                             wait for PERIOD;  
                             sys_data_i <= x"3210";
      wait for (PERIOD);     sys_cmd <= cmd_sys_nop; --NOP;
      wait for (10*PERIOD);  sys_cmd <= cmd_sys_read; --READ_AUTOPRECHARGE
                             --{ba,row_addr,col_addr} = {2'b00, 12'h200, 8'h10}
                             sys_addr <= "00" & x"200" & x"10";
      wait for (PERIOD);     sys_cmd <= cmd_sys_nop; --NOP;
      wait for (19*PERIOD);  sys_cmd <= cmd_sys_read; --READ_AUTOPRECHARGE
                             --{ba,row_addr,col_addr} = {2'b01, 12'h200, 8'h20}
                             sys_addr <= "01" & x"200" & x"20";
      wait for (PERIOD);     sys_cmd <= cmd_sys_nop; --NOP;
      wait for (19*PERIOD);
  end procedure;
  
  begin

    sys_cmd <= cmd_sys_nop;
    sys_addr <= "0000000000000000000000";      
    sys_data_i <= x"0000";
    --sys_clk <= '0';
    sys_reset_n <= '1';
    sys_cmd(SYS_NOP) <= '1';
    
    wait for (32*PERIOD);   --wait for DLL in FPGA to lock
    wait for (HALF_PERIOD); --delay inputs to avoid race condition
    wait for (PERIOD);     sys_cmd <= cmd_sys_nop; --NOP;
    wait for (DELAY_TIME); sys_cmd <= cmd_sys_precharge; --PRECHARGE_ALL_BANKS;
    wait for (PERIOD);     sys_cmd <= cmd_sys_nop; --NOP;
    wait for (PERIOD);     sys_cmd <= cmd_sys_load_mr;--EMRS;
                           sys_addr <= "0100000000000000000000";--sys_ad(20)=1

    run_test ("011", "010"); -- burst = 8, cas latency = 2;
    run_test ("011", "110"); -- burst = 8, cas latency = 2.5;
              
    run_test ("010", "010"); -- burst = 4, cas latency = 2;
    run_test ("010", "110"); -- burst = 4, cas latency = 2.5;
              
    run_test ("001", "010"); -- burst = 2, cas latency = 2;
    run_test ("001", "110"); -- burst = 2, cas latency = 2.5;

    end process;

    -- purpose : decode ddr commands 
    -- type    : combinational
    -- inputs  : ddr_rasb,ddr_casb,ddr_web
    -- outputs : DDR_COMMANDS
    
    ddr_controls <= ddr_rasb & ddr_casb & ddr_web;

    DDR_CMD_DECODE : process (ddr_rasb,ddr_casb,ddr_web)
      variable DDR_COMMANDS : ddr_cmd;
    begin  -- process DDR_CMD_DECODE
      case ddr_controls is
        when "000" => DDR_COMMANDS := LOAD_MR;
        when "001" => DDR_COMMANDS := AUTO_REFRESH;
        when "010" => DDR_COMMANDS := PRECHARGE;
        when "011" => DDR_COMMANDS := ACT;
        when "100" => DDR_COMMANDS := WRITEA;
        when "101" => DDR_COMMANDS := READA;
        when "110" => DDR_COMMANDS := BURST_STOP;
        when "111" => DDR_COMMANDS := NOP;
        when others => DDR_COMMANDS := UNKNOWN;
      end case;
    end process DDR_CMD_DECODE;
  end test_ARCH;

configuration test_CONF of test_E is
  for test_ARCH
  end for;  
end test_CONF;



