// Author:     
// Created by: /tools/cheshire/denali/verify/tools/bin/cr_tb.perl version 1.02
// Date:       Mon Feb  1 17:26:26 PST 1999
// File:       tb_ddr_sdram.v
// Comment:
// need to keep sys_cmd READ & WRITE for several periods to generate burst_end


`include "../src/define.v"
`include "../ram_model/mt46v4m16-133.v"

`timescale 1ns / 1ps

//133MHz
`define PERIOD       7.5
`define HALF_PERIOD  3.75
`define DELAY_TIME   0.7

`define sys_load_mr            5'b00001
`define sys_read               5'b00010
`define sys_write              5'b00100
`define sys_precharge          5'b01000
`define sys_refresh            5'b10000
`define sys_nop                5'b00000

module test();
   
   reg [`SYS_ADDR_MSB:0] sys_addr;
   reg [6:2] 		 sys_cmd;
   reg 			 sys_reset_n;
   reg 			 sys_clk;
   reg [15:0] 		 sys_data_i;
   
   wire [`DDR_ADDR_MSB:0] ddr_ad;
   wire [63:0] 		  ddr_dq; 
   wire [1:0] 		  ddr_dm;
   wire [1:0] 		  ddr_ba;
   wire [3:0] 		  ddr_dqs;
   wire 		  ddr_rasb;
   wire 		  ddr_casb;
   wire 		  ddr_web;

   wire [15:0] 		  sys_data_o;
   
   reg 			  GSR;
   
`include "string_decode_ba.v"
   
   Synopsys_edif utt(
//   top utt(
	   .sys_reset_n(sys_reset_n), .sys_ref_ack(sys_ref_ack), 
	   .sys_data_valid(sys_data_valid), .u_clk_fb(sys_clk_fb), 
	   .u_clk(sys_clk), .ddr_web(ddr_web), 
	   .ddr_rasb(ddr_rasb), .ddr_csb(ddr_csb), 
	   .ddr_clkb(ddr_clkb), .ddr_clk(ddr_clk), 
	   .ddr_cke(ddr_cke), .ddr_casb(ddr_casb), 
	   .sys_data_o(sys_data_o), .sys_data_i(sys_data_i), 
	   .sys_cmd(sys_cmd), .sys_addr(sys_addr), 
	   .ddr_dq(ddr_dq), .ddr_dqs(ddr_dqs), 
	   .ddr_dm(ddr_dm), .ddr_ba(ddr_ba), 
	   .ddr_ad(ddr_ad)
  );
   

   assign  sys_clk_fb = ddr_clk;  //feed DDR clock back to FPGA 
   wire    #(0.5) ddr_clk_dly =  ddr_clk;
   wire    #(0.5) ddr_clkb_dly =  ddr_clkb;
   
   mt46v4m16 u_ddr1 (.Clk(ddr_clk_dly), .Clk_n(ddr_clkb_dly), .Cs_n(ddr_csb), 
	      .Cke(ddr_cke), .Ba(ddr_ba), .Addr(ddr_ad), 
	      .Ras_n(ddr_rasb), .Cas_n(ddr_casb), .We_n(ddr_web), 
	      .Dm(ddr_dm), .Dq(ddr_dq[15:0]), .Dqs(ddr_dqs[0]));
   mt46v4m16 u_ddr2 (.Clk(ddr_clk_dly), .Clk_n(ddr_clkb_dly), .Cs_n(ddr_csb), 
	      .Cke(ddr_cke), .Ba(ddr_ba), .Addr(ddr_ad), 
	      .Ras_n(ddr_rasb), .Cas_n(ddr_casb), .We_n(ddr_web), 
	      .Dm(ddr_dm), .Dq(ddr_dq[31:16]), .Dqs(ddr_dqs[1]));
   mt46v4m16 u_ddr3 (.Clk(ddr_clk_dly), .Clk_n(ddr_clkb_dly), .Cs_n(ddr_csb), 
	      .Cke(ddr_cke), .Ba(ddr_ba), .Addr(ddr_ad), 
	      .Ras_n(ddr_rasb), .Cas_n(ddr_casb), .We_n(ddr_web), 
	      .Dm(ddr_dm), .Dq(ddr_dq[47:32]), .Dqs(ddr_dqs[2]));
    mt46v4m16 u_ddr4 (.Clk(ddr_clk_dly), .Clk_n(ddr_clkb_dly), .Cs_n(ddr_csb), 
	      .Cke(ddr_cke), .Ba(ddr_ba), .Addr(ddr_ad), 
	      .Ras_n(ddr_rasb), .Cas_n(ddr_casb), .We_n(ddr_web), 
	      .Dm(ddr_dm), .Dq(ddr_dq[63:48]), .Dqs(ddr_dqs[3]));
 
   always #(`HALF_PERIOD) sys_clk <= ~sys_clk;
   

   initial begin
      sys_cmd = `sys_nop;
      sys_addr = 22'h0_0000;
      sys_data_i = 16'h0000;
      sys_clk = 0;
      sys_reset_n = 1;
      
      #(32*`PERIOD);   //wait for DLL in FPGA to lock
      #(`HALF_PERIOD); //delay inputs to avoid race condition
      #(`PERIOD);     nop;
      #(`DELAY_TIME); precharge_all_banks;
      #(`PERIOD);     nop;
      #(`PERIOD);     EMRS;
      run_test(3'b011,3'b010); // burst = 8, cas latency = 2;
      run_test(3'b011,3'b110); // burst = 8, cas latency = 2.5;
		     
      run_test(3'b010,3'b010); // burst = 4, cas latency = 2;
      run_test(3'b010,3'b110); // burst = 4, cas latency = 2.5;
		     
      run_test(3'b001,3'b010); // burst = 2, cas latency = 2;
      run_test(3'b001,3'b110); // burst = 2, cas latency = 2.5;
      $finish;
   end // initial begin
   
   task run_test;
      input [2:0] burst_length;
      input [2:0] cas_latency;
      begin      
	 #(`PERIOD);     nop;
	 #(`PERIOD);     MRS(burst_length,0,cas_latency); 
	 #(`PERIOD);     nop;
	 #(`PERIOD);     precharge_all_banks;
	 #(`PERIOD);     nop;
	 #(2*`PERIOD);   auto_refresh;
	 #(`PERIOD);     nop;
	 #(8*`PERIOD);   auto_refresh;
	 #(`PERIOD);     nop;
	 //write_autoprecharge (ba,row_addr,col_addr,row_addr,ba,write_data1,write_data2,write_data3,write_data4)
	 #(8*`PERIOD);   write_autoprecharge(2'b00, 12'h200, 8'h10, 16'h1020, 16'h0304, 16'h5060, 16'h0708);
	 #(`PERIOD);     nop;
	 #(10*`PERIOD);  read_autoprecharge(2'b00, 12'h200, 8'h10);
	 #(`PERIOD);   nop;
	 #(10*`PERIOD);  write_autoprecharge(2'b01, 12'h200, 8'h20, 16'h0987, 16'h9876, 16'h7654, 16'h3210);
	 #(`PERIOD);     nop;
	 #(10*`PERIOD);  read_autoprecharge(2'b00, 12'h200, 8'h10);
	 #(`PERIOD);   nop;
	 #(10*`PERIOD);  read_autoprecharge(2'b01, 12'h200, 8'h20);
	 #(`PERIOD);   nop;
	 #(10*`PERIOD);
      end
   endtask
   
   task EMRS;
      begin
	 sys_cmd = `sys_load_mr;
	 sys_addr = {22'h100000};  //ba0 = 1, sys_ad[20] = 1
      end
   endtask // EMRS   
   task MRS;
      input [2:0] burst_length;
      input 	     burst_type;
      input [2:0] cas_latency;
      begin
	 sys_cmd = `sys_load_mr;
	 sys_addr = {12'h000,1'b0,cas_latency,burst_type,burst_length}; //ba0 = 0, sys_ad[20] = 0 
      end
   endtask // MRS
   task auto_refresh;
      begin
	 sys_cmd = `sys_refresh;
	 #(`PERIOD);
	 sys_cmd = `sys_nop;
      end
   endtask // auto_refresh
   task read_autoprecharge;
      input [1:0]  ba;
      input [`ROW_ADDR_MSB:0] row_addr;
      input [`COL_ADDR_MSB:0] col_addr;
      begin
	 sys_cmd = `sys_read;
	 sys_addr = {ba,row_addr,col_addr};
	 //$display("tb:in read task, ba = %h, row_addr = %h, col_addr = %h, sys_ad = %h", ba, row_addr, col_addr, sys_addr);
	 #(`PERIOD);
	 sys_cmd = `sys_nop;
	 #(9*`PERIOD);
      end
   endtask // read_autoprecharge
   task write_autoprecharge; //for burst length = 4
      input [1:0]  ba;
      input [`ROW_ADDR_MSB:0] row_addr;
      input [`COL_ADDR_MSB:0] col_addr;
      input [15:0] write_data1,write_data2,write_data3,write_data4;
      //write cmd
      begin	 
	 sys_cmd = `sys_write;
	 sys_addr = {ba,row_addr,col_addr};
	 //$display("tb: in write task, ba = %h, row_addr = %h, col_addr = %h, sys_ad = %h", ba, row_addr, col_addr, sys_addr);
	 //#(5*`PERIOD);
	 sys_data_i = write_data1;
	 #(`PERIOD);  
	 sys_data_i = write_data2;
	 #(`PERIOD);  
	 sys_data_i = write_data3;
	 #(`PERIOD);  
	 sys_data_i = write_data4;
      end
   endtask // write_autoprecharge
   task burst_stop;
      sys_cmd = `sys_burst_stop;
   endtask // burst_stop

   task precharge_all_banks;
      begin      
	 sys_cmd = `sys_precharge;
      end
   endtask // precharge
   task nop;
      sys_cmd = `sys_nop;
   endtask // nop
   

   //for virsim
   initial $vcdpluson(test);
   

endmodule
 