-----------------------------------------------------------------------------------------
--
--     File Name: MT46V4M16.VHD
--       Version: 0.0f
--          Date: May 24th, 1999
--         Model: Behavioral
--     Simulator: Model Technology VLOG (PC version 5.2e PE)
--
--  Dependencies: None
--
--        Author: Son P. Huynh
--         Email: sphuynh@micron.com
--       Company: Micron Technology, Inc.
--   Part Number: MT46V4M16 (1 Mb x 16 x 4 Banks)
--
--   Description: Micron 64 Mb SDRAM DDR (Double Data Rate)
--
--    Limitation: - Doesn't check for 4096-cycle refresh
--
--          Note: - Set simulator resolution to "ps" accuracy
--                - Model assume Clk and Clk# crossing at both edge
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER AND MICRON SPECIFICALLY DISCLAIMS ANY 
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (c) 1998 Micron Semiconductor Products, Inc.
--                All rights researved
--
--  Rev   Author                        Date        Changes
--  ----  ----------------------------  ----------  -------------------------------------
--  0.0f  Son P. Huynh                  05/24/1999  Fix DQS not properly HiZ after
--        Micron Technology, Inc.                     precharge or burst terminate.
--                                                  Fix DQS not properly go LOW if Read
--                                                    immediate after precharge or burst term 
--                                                  Add detection for interrupting Read
--                                                    or Write with Auto Precharge.      
--
--  0.0d  Son P. Huynh                  05/13/1999  First Release (from MT46V8M8.VHD)
--        Micron Technology, Inc.                   - Now using VHDL 1076-93 (SHARED)
--                                                  - New testbench
-----------------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
LIBRARY work;
USE work.mti_pkg.ALL;
package mt46v4m16_pkg is
  component mt46v4m16
    generic(
      addr_bits     : INTEGER := 12;
      clk_start     : time    :=  5 ns;
      clk_period    : time    := 10 ns;
      continue_time : time    := 10 ns;
      data_bits     : INTEGER := 16
      );

    PORT (
      Dq    : INOUT STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0) := (OTHERS => 'Z');
      Dqs   : INOUT STD_LOGIC_VECTOR (1 DOWNTO 0) := "ZZ";
      Addr  : IN    STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0) := (OTHERS => '0');
      Ba    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0) := "00";
      Clk   : IN    STD_LOGIC := '0';
      Clk_n : IN    STD_LOGIC := '1';
      Cke   : IN    STD_LOGIC := '0';
      Cs_n  : IN    STD_LOGIC := '1';
      Ras_n : IN    STD_LOGIC := '0';
      Cas_n : IN    STD_LOGIC := '0';
      We_n  : IN    STD_LOGIC := '0';
      Dm    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0) := "00"
    );  
  END component;
end mt46v4m16_pkg;

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
LIBRARY work;
USE work.mti_pkg.ALL;
ENTITY mt46v4m16 IS
    generic(
      addr_bits     : INTEGER := 12;
      clk_start     : time    :=  5 ns;
      clk_period    : time    := 10 ns;
      continue_time : time    := 10 ns;
      data_bits     : INTEGER := 16
      );
    PORT (
        Dq    : INOUT STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0) := (OTHERS => 'Z');
        Dqs   : INOUT STD_LOGIC_VECTOR (1 DOWNTO 0) := "ZZ";
        Addr  : IN    STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0) := (OTHERS => '0');
        Ba    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0) := "00";
        Clk   : IN    STD_LOGIC := '0';
        Clk_n : IN    STD_LOGIC := '1';
        Cke   : IN    STD_LOGIC := '0';
        Cs_n  : IN    STD_LOGIC := '1';
        Ras_n : IN    STD_LOGIC := '0';
        Cas_n : IN    STD_LOGIC := '0';
        We_n  : IN    STD_LOGIC := '0';
        Dm    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0) := "00"
    );
END mt46v4m16;

ARCHITECTURE behave OF mt46v4m16 IS
    TYPE   State       IS (ACT, A_REF, BST, EMR, LMR, NOP, PRECH, READ, READ_A, WRITE, WRITE_A);
    TYPE   Array4xI    IS ARRAY (3 DOWNTO 0) OF INTEGER;
    TYPE   Array4xB    IS ARRAY (3 DOWNTO 0) OF BIT;
    TYPE   Array7xB    IS ARRAY (6 DOWNTO 0) OF BIT;
    TYPE   Array7x2BV  IS ARRAY (6 DOWNTO 0) OF BIT_VECTOR (1 DOWNTO 0);
    TYPE   Array7xCBV  IS ARRAY (6 DOWNTO 0) OF BIT_VECTOR (col_bits - 1 DOWNTO 0);
    TYPE   Array_state IS ARRAY (6 DOWNTO 0) OF State;
    SIGNAL Operation, Rw_command : State := NOP;

    SIGNAL Bank, Bank_dqs, Previous_bank : BIT_VECTOR (1 DOWNTO 0) := "00";
    SIGNAL Row, Row_dqs : BIT_VECTOR (addr_bits - 1 DOWNTO 0) := (OTHERS => '0');
    SIGNAL Col_dqs : BIT_VECTOR (col_bits - 1 DOWNTO 0) := (OTHERS => '0');
    SIGNAL Col_brst, Col_brst_dqs : BIT_VECTOR (col_bits - 1 DOWNTO 0) := (OTHERS => '0');

    SIGNAL B0_row_addr, B1_row_addr, B2_row_addr, B3_row_addr : BIT_VECTOR (addr_bits - 1 DOWNTO 0) := (OTHERS => '0');

    SHARED VARIABLE Burst_counter : INTEGER := 0;

    SHARED VARIABLE Data_in_enable : BIT := '0';
    SHARED VARIABLE Data_out_enable : BIT := '0';

    SHARED VARIABLE Col : BIT_VECTOR (col_bits - 1 DOWNTO 0) := (OTHERS => '0');
    SHARED VARIABLE Col_addr : Array7xCBV;
    SHARED VARIABLE Bank_addr : Array7x2BV;

    SHARED VARIABLE WR_chk  : Array4xI;

    SIGNAL Mode_reg : BIT_VECTOR (addr_bits - 1 DOWNTO 0) := (OTHERS => '0');
    SIGNAL Active_enable, Aref_enable, Burst_term, Ext_mode_enable : BIT := '0';
    SIGNAL Mode_reg_enable, Prech_enable, Read_enable, Write_enable : BIT := '0';
    SIGNAL Burst_length_2, Burst_length_4, Burst_length_8 : BIT := '0';
    SIGNAL Cas_latency_2, Cas_latency_25, Cas_latency_3 : BIT := '0';
    SIGNAL Ras_in, Cas_in, We_in : BIT := '0';
    SIGNAL Sys_clk, CkeZ : BIT := '0';

    SIGNAL Dqs_int, Dqs_out : STD_LOGIC_VECTOR (1 DOWNTO 0) := "ZZ";

    -- Checking internal wires
    SIGNAL Pre_chk : BIT_VECTOR (3 DOWNTO 0) := "0000";
    SIGNAL Act_chk : BIT_VECTOR (3 DOWNTO 0) := "0000";

BEGIN
    -- CS# Decode
    WITH Cs_n SELECT
        Cas_in <= TO_BIT (Cas_n, '1') WHEN '0',
                  '1' WHEN '1',
                  '1' WHEN OTHERS;
    WITH Cs_n SELECT
        Ras_in <= TO_BIT (Ras_n, '1') WHEN '0',
                  '1' WHEN '1',
                  '1' WHEN OTHERS;
    WITH Cs_n SELECT
        We_in  <= TO_BIT (We_n,  '1') WHEN '0',
                  '1' WHEN '1',
                  '1' WHEN OTHERS;
    
    -- Commands Decode
    Active_enable   <= NOT(Ras_in) AND     Cas_in  AND     We_in;
    Aref_enable     <= NOT(Ras_in) AND NOT(Cas_in) AND     We_in;
    Burst_term      <=     Ras_in  AND     Cas_in  AND NOT(We_in);
    Ext_mode_enable <= NOT(Ras_in) AND NOT(Cas_in) AND NOT(We_in) AND     TO_BIT(Ba(0))  AND NOT(TO_BIT(Ba(1)));
    Mode_reg_enable <= NOT(Ras_in) AND NOT(Cas_in) AND NOT(We_in) AND NOT(TO_BIT(Ba(0))) AND NOT(TO_BIT(Ba(1))) AND TO_BIT(Addr(5));
    Prech_enable    <= NOT(Ras_in) AND     Cas_in  AND NOT(We_in);
    Read_enable     <=     Ras_in  AND NOT(Cas_in) AND     We_in;
    Write_enable    <=     Ras_in  AND NOT(Cas_in) AND NOT(We_in);

    -- Burst Length Decode
    Burst_length_2  <= NOT(Mode_reg(2)) AND NOT(Mode_reg(1)) AND     Mode_reg(0);
    Burst_length_4  <= NOT(Mode_reg(2)) AND     Mode_reg(1)  AND NOT(Mode_reg(0));
    Burst_length_8  <= NOT(Mode_reg(2)) AND     Mode_reg(1)  AND     Mode_reg(0);

    -- CAS Latency Decode
    Cas_latency_2   <= NOT(Mode_reg(6)) AND     Mode_reg(5)  AND NOT(Mode_reg(4));
    Cas_latency_25  <=     Mode_reg(6)  AND     Mode_reg(5)  AND NOT(Mode_reg(4));
    Cas_latency_3   <= NOT(Mode_reg(6)) AND     Mode_reg(5)  AND     Mode_reg(4);

    Dqs             <= Dqs_out;

    -- System Clock
    int_clk : PROCESS (Clk, Clk_n)
        VARIABLE ClkZ : BIT := '0';
    begin
        IF Clk = '1' AND Clk_n = '0' THEN
            ClkZ := '1';
            CkeZ <= TO_BIT(Cke, '1');
        ELSIF Clk = '0' AND Clk_n = '1' THEN
            ClkZ := '0';
        END IF;
        Sys_clk <= CkeZ AND ClkZ;
    END PROCESS;

    state_register : PROCESS
        VARIABLE Command : Array_state;
        VARIABLE Dll_enable : BIT := '0';

        VARIABLE Pc_b0, Pc_b1, Pc_b2, Pc_b3 : BIT := '0';
        VARIABLE Act_b0, Act_b1, Act_b2, Act_b3 : BIT := '0';

        VARIABLE Bank_precharge  : Array7x2BV;
        VARIABLE A10_precharge   : Array7xB;
        VARIABLE Auto_precharge  : Array4xB;
        VARIABLE Read_precharge  : Array4xB;
        VARIABLE Write_precharge : Array4xB;
        VARIABLE Count_precharge : Array4xI;

        -- Timing Check
        VARIABLE DLL_chk : INTEGER := 0;
        VARIABLE MRD_chk : TIME := 0 ns;
        VARIABLE RC_chk, RRD_chk : TIME := 0 ns;
        VARIABLE RAS_chk0, RAS_chk1, RAS_chk2, RAS_chk3 : TIME := 0 ns;
        VARIABLE RCD_chk0, RCD_chk1, RCD_chk2, RCD_chk3 : TIME := 0 ns;
        VARIABLE RP_chk, RP_chk0, RP_chk1, RP_chk2, RP_chk3 : TIME := 0 ns;

    BEGIN
        WAIT ON Sys_clk;
        -- Internal Command Pipeline
        Command(0) := Command(1);
        Command(1) := Command(2);
        Command(2) := Command(3);
        Command(3) := Command(4);
        Command(4) := Command(5);
        Command(5) := Command(6);
        Command(6) := NOP;

        Col_addr(0) := Col_addr(1);
        Col_addr(1) := Col_addr(2);
        Col_addr(2) := Col_addr(3);
        Col_addr(3) := Col_addr(4);
        Col_addr(4) := Col_addr(5);
        Col_addr(5) := Col_addr(6);
        Col_addr(6) := (OTHERS => '0');

        Bank_addr(0) := Bank_addr(1);
        Bank_addr(1) := Bank_addr(2);
        Bank_addr(2) := Bank_addr(3);
        Bank_addr(3) := Bank_addr(4);
        Bank_addr(4) := Bank_addr(5);
        Bank_addr(5) := Bank_addr(6);
        Bank_addr(6) := (OTHERS => '0');

        -- Precharge Pipeline
        Bank_precharge(0) := Bank_precharge(1);
        Bank_precharge(1) := Bank_precharge(2);
        Bank_precharge(2) := Bank_precharge(3);
        Bank_precharge(3) := Bank_precharge(4);
        Bank_precharge(4) := Bank_precharge(5);
        Bank_precharge(5) := Bank_precharge(6);
        Bank_precharge(6) := (OTHERS => '0');

        A10_precharge(0) := A10_precharge(1);
        A10_precharge(1) := A10_precharge(2);
        A10_precharge(2) := A10_precharge(3);
        A10_precharge(3) := A10_precharge(4);
        A10_precharge(4) := A10_precharge(5);
        A10_precharge(5) := A10_precharge(6);
        A10_precharge(6) := '0';

        -- tWR Counter
        WR_chk(0) := WR_chk(0) + 1;
        WR_chk(1) := WR_chk(1) + 1;
        WR_chk(2) := WR_chk(2) + 1;
        WR_chk(3) := WR_chk(3) + 1;

        -- Commands Operation Decode
        IF Sys_clk = '1' THEN
            -- Operation Decode
            IF Active_enable = '1' THEN
                Operation <= ACT;
            ELSIF Aref_enable = '1' THEN
                Operation <= A_REF;
            ELSIF Burst_term = '1' THEN
                Operation <= BST;
            ELSIF Ext_mode_enable = '1' THEN
                Operation <= EMR;
            ELSIF Mode_reg_enable = '1' THEN
                Operation <= LMR;
            ELSIF Prech_enable = '1' THEN
                Operation <= PRECH;
            ELSIF Read_enable = '1' THEN
                IF Addr(10) = '0' THEN
                    Operation <= READ;
                ELSE
                    Operation <= READ_A;
                END IF;
            ELSIF Write_enable = '1' THEN
                IF Addr(10) = '0' THEN
                    Operation <= WRITE;
                ELSE
                    Operation <= WRITE_A;
                END IF;
            ELSE
                Operation <= NOP;
            END IF;

            -- Read or Write with Auto Precharge Counter
            IF Auto_precharge (0) = '1' THEN
                Count_precharge (0) := Count_precharge (0) + 1;
            END IF;
            IF Auto_precharge (1) = '1' THEN
                Count_precharge (1) := Count_precharge (1) + 1;
            END IF;
            IF Auto_precharge (2) = '1' THEN
                Count_precharge (2) := Count_precharge (2) + 1;
            END IF;
            IF Auto_precharge (3) = '1' THEN
                Count_precharge (3) := Count_precharge (3) + 1;
            END IF;

            -- tMRD Counter
            MRD_chk := MRD_chk + tCK;

            -- Auto Refresh
            IF Aref_enable = '1' THEN
                -- Auto Refresh to Auto Refresh
                ASSERT (NOW - RC_chk >= tRC)
                    REPORT "tRC violation during Auto Refresh"
                    SEVERITY WARNING;
                -- Precharge to Auto Refresh
                ASSERT (NOW - RP_chk >= tRP)
                    REPORT "tRP violation during Auto Refresh"
                    SEVERITY WARNING;
                -- Precharge to Auto Refresh
                ASSERT (Pc_b0 = '1' AND Pc_b1 = '1' AND Pc_b2 = '1' AND Pc_b3 = '1')
                    REPORT "All banks must be Precharge before Auto Refresh"
                    SEVERITY WARNING;
                -- Record current tRC time
                RC_chk := NOW;
            END IF;

            -- Extended Load Mode Register
            IF Ext_mode_enable = '1' THEN
                IF (Pc_b0 = '1' AND Pc_b1 = '1' AND Pc_b2 = '1' AND Pc_b3 = '1') THEN
                    IF Addr(0) = '0' THEN
                        Dll_enable := '1';
                    ELSE
                        Dll_enable := '0';
                    END IF;
                    -- LMR/EMR to EMR
                    ASSERT (MRD_chk >= tMRD)
                        REPORT "tMRD violation during Extended Mode Register"
                        SEVERITY WARNING;
                ELSE
                    ASSERT (FALSE)
                        REPORT "All bank must be Precharge before Extended Mode Register"
                        SEVERITY WARNING;
                END IF;
                -- Record current tMRD time
                MRD_chk := 0 ns;
            END IF;

            -- Load Mode Register
            IF Mode_reg_enable = '1' THEN
                Mode_reg <= TO_BITVECTOR (Addr);
                IF (Pc_b0 /= '1' OR Pc_b1 /= '1' OR Pc_b2 /= '1' OR Pc_b3 /= '1') THEN
                    ASSERT (FALSE)
                        REPORT "All bank must be Precharge before Load Mode Register"
                        SEVERITY WARNING;
                END IF;
                -- LMR/ELMR to LMR
                ASSERT (MRD_chk >= tMRD)
                    REPORT "tMRD violation during Load Mode Register"
                    SEVERITY WARNING;
                -- Record current tMRD time
                MRD_chk := 0 ns;
            END IF;

            -- Active Block (latch Bank and Row Address)
            IF Active_enable = '1' THEN
                IF Ba = "00" AND Pc_b0 = '1' THEN
                    Act_b0 := '1';
                    Pc_b0 := '0';
                    B0_row_addr <= TO_BITVECTOR (Addr);
                    RCD_chk0 := NOW;
                    RAS_chk0 := NOW;
                    -- Precharge to Active Bank 0
                    ASSERT (NOW - RP_chk0 >= tRP)
                        REPORT "tRP violation during Activate Bank 0"
                        SEVERITY WARNING;
                ELSIF Ba = "01" AND Pc_b1 = '1' THEN
                    Act_b1 := '1';
                    Pc_b1 := '0';
                    B1_row_addr <= TO_BITVECTOR (Addr);
                    RCD_chk1 := NOW;
                    RAS_chk1 := NOW;
                    -- Precharge to Active Bank 1
                    ASSERT (NOW - RP_chk1 >= tRP)
                        REPORT "tRP violation during Activate Bank 1"
                        SEVERITY WARNING;
                ELSIF Ba = "10" AND Pc_b2 = '1' THEN
                    Act_b2 := '1';
                    Pc_b2 := '0';
                    B2_row_addr <= TO_BITVECTOR (Addr);
                    RCD_chk2 := NOW;
                    RAS_chk2 := NOW;
                    -- Precharge to Active Bank 2
                    ASSERT (NOW - RP_chk2 >= tRP)
                        REPORT "tRP violation during Activate Bank 2"
                        SEVERITY WARNING;
                ELSIF Ba = "11" AND Pc_b3 = '1' THEN
                    Act_b3 := '1';
                    Pc_b3 := '0';
                    B3_row_addr <= TO_BITVECTOR (Addr);
                    RCD_chk3 := NOW;
                    RAS_chk3 := NOW;
                    -- Precharge to Active Bank 3
                    ASSERT (NOW - RP_chk3 >= tRP)
                        REPORT "tRP violation during Activate Bank 3"
                        SEVERITY WARNING;
                ELSIF Ba = "00" AND Pc_b0 = '0' THEN
                    ASSERT (FALSE)
                        REPORT "Bank 0 is not Precharged"
                        SEVERITY WARNING;
                ELSIF Ba = "01" AND Pc_b1 = '0' THEN
                    ASSERT (FALSE)
                        REPORT "Bank 1 is not Precharged"
                        SEVERITY WARNING;
                ELSIF Ba = "10" AND Pc_b2 = '0' THEN
                    ASSERT (FALSE)
                        REPORT "Bank 2 is not Precharged"
                        SEVERITY WARNING;
                ELSIF Ba = "11" AND Pc_b3 = '0' THEN
                    ASSERT (FALSE)
                        REPORT "Bank 3 is not Precharged"
                        SEVERITY WARNING;
                END IF;
                -- Activate Bank A to Activate Bank B
                IF (Previous_Bank /= TO_BITVECTOR(Ba)) AND (NOW - RRD_chk < tRRD) THEN
                    ASSERT (FALSE)
                        REPORT "tRRD violation during Activate"
                        SEVERITY WARNING;
                END IF;
                -- AutoRefresh to Activate
                ASSERT (NOW - RC_chk >= tRC)
                    REPORT "tRC violation during Activate"
                    SEVERITY WARNING;
                -- Record Variables for Checking Violation
                RRD_chk := NOW;
                Previous_bank <= TO_BITVECTOR(Ba);
            END IF;

            -- Precharge Block
            IF Prech_enable = '1' THEN
                IF Addr(10) = '1' THEN
                    Pc_b0 := '1';
                    Pc_b1 := '1';
                    Pc_b2 := '1';
                    Pc_b3 := '1';
                    Act_b0 := '0';
                    Act_b1 := '0';
                    Act_b2 := '0';
                    Act_b3 := '0';
                    RP_chk0 := NOW;
                    RP_chk1 := NOW;
                    RP_chk2 := NOW;
                    RP_chk3 := NOW;
                    -- Activate to Precharge all banks
                    ASSERT ((NOW - RAS_chk0 >= tRAS) OR (NOW - RAS_chk1 >= tRAS) OR
                            (NOW - RAS_chk2 >= tRAS) OR (NOW - RAS_chk3 >= tRAS))
                        REPORT "tRAS violation during Precharge all banks"
                        SEVERITY WARNING;
                ELSIF Addr(10) = '0' THEN
                    IF Ba = "00" THEN
                        Pc_b0 := '1';
                        Act_b0 := '0';
                        RP_chk0 := NOW;
                        -- Activate to Precharge bank 0
                        ASSERT (NOW - RAS_chk0 >= tRAS)
                            REPORT "tRAS violation during Precharge bank 0"
                            SEVERITY WARNING;
                    ELSIF Ba = "01" THEN
                        Pc_b1 := '1';
                        Act_b1 := '0';
                        RP_chk1 := NOW;
                        -- Activate to Precharge bank 1
                        ASSERT (NOW - RAS_chk1 >= tRAS)
                            REPORT "tRAS violation during Precharge bank 1"
                            SEVERITY WARNING;
                    ELSIF Ba = "10" THEN
                        Pc_b2 := '1';
                        Act_b2 := '0';
                        RP_chk2 := NOW;
                        -- Activate to Precharge bank 2
                        ASSERT (NOW - RAS_chk2 >= tRAS)
                            REPORT "tRAS violation during Precharge bank 2"
                            SEVERITY WARNING;
                    ELSIF Ba = "11" THEN
                        Pc_b3 := '1';
                        Act_b3 := '0';
                        RP_chk3 := NOW;
                        -- Activate to Precharge bank 3
                        ASSERT (NOW - RAS_chk3 >= tRAS)
                            REPORT "tRAS violation during Precharge bank 3"
                            SEVERITY WARNING;
                    END IF;

                    -- tWR violation check
                    ASSERT (WR_chk(TO_INTEGER(Ba)) > tWR)
                        REPORT "tWR violation during Precharge"
                        SEVERITY WARNING;
                END IF;

                -- Terminate a Write Immediately (if same bank or all banks)
                IF (Data_in_enable = '1' AND (Bank = TO_BITVECTOR(Ba) OR Addr(10) = '1')) THEN
                    Data_in_enable := '0';
                END IF;

                -- Precharge Command Pipeline for READ
                IF CAS_latency_3 = '1' THEN
                    Command(6) := PRECH;
                    Bank_precharge(6) := TO_BITVECTOR (Ba);
                    A10_precharge(6) := TO_BIT(Addr(10));
                ELSIF CAS_latency_25 = '1' THEN
                    Command(5) := PRECH;
                    Bank_precharge(5) := TO_BITVECTOR (Ba);
                    A10_precharge(5) := TO_BIT(Addr(10));
                ELSIF CAS_latency_2 = '1' THEN
                    Command(4) := PRECH;
                    Bank_precharge(4) := TO_BITVECTOR (Ba);
                    A10_precharge(4) := TO_BIT(Addr(10));
                END IF;
                -- Record Current tRP time
                RP_chk := NOW;
            END IF;

            -- Burst Terminate
            IF Burst_term = '1' THEN
                IF CAS_latency_3 = '1' THEN
                    Command(6) := BST;
                ELSIF CAS_latency_25 = '1' THEN
                    Command(5) := BST;
                ELSIF CAS_latency_2 = '1' THEN
                    Command(4) := BST;
                END IF;
                IF (Data_in_enable = '1' AND Write_precharge(TO_INTEGER(Bank)) = '1') OR
                   (Data_out_enable = '1' AND Read_precharge(TO_INTEGER(Bank)) = '1') THEN
                    ASSERT (FALSE)
                        REPORT "It's illegal to terminate a Read or Write with Auto Precharge"
                        SEVERITY WARNING;
                END IF;
            END IF;

            -- Read, Write, Column Latch
            IF Read_enable = '1' OR Write_enable = '1' THEN
                -- Check to see if bank is open (ACT) for Read or Write
                IF ((Ba = "00" AND Pc_b0 = '1') OR (Ba = "01" AND Pc_b1 = '1') OR
                    (Ba = "10" AND Pc_b2 = '1') OR (Ba = "11" AND Pc_b3 = '1')) THEN
                    ASSERT (FALSE)
                        REPORT "Cannot Read or Write - Bank is not Activated"
                        SEVERITY WARNING;
                END IF;

                -- Activate to Read or Write
                IF Ba = "00" THEN
                    ASSERT (NOW - RCD_chk0 >= tRCD)
                        REPORT "tRCD violation during Read or Write to Bank 0"
                        SEVERITY WARNING;
                ELSIF Ba = "01" THEN
                    ASSERT (NOW - RCD_chk1 >= tRCD)
                        REPORT "tRCD violation during Read or Write to Bank 1"
                        SEVERITY WARNING;
                ELSIF Ba = "10" THEN
                    ASSERT (NOW - RCD_chk2 >= tRCD)
                        REPORT "tRCD violation during Read or Write to Bank 2"
                        SEVERITY WARNING;
                ELSIF Ba = "11" THEN
                    ASSERT (NOW - RCD_chk3 >= tRCD)
                        REPORT "tRCD violation during Read or Write to Bank 3"
                        SEVERITY WARNING;
                END IF;

                -- Interrupting a Read or Write with Auto Precharge (same bank only)
                IF (Auto_precharge(To_Integer(Ba)) = '1' AND (Read_precharge(To_Integer(Ba)) = '1' OR Write_precharge(To_Integer(Ba)) = '1')) THEN
                     ASSERT (FALSE)
                        REPORT "It's illegal to interrupt a Read or Write with Auto Precharge "
                        SEVERITY WARNING;
                END IF;
                    
                -- Read Command
                IF Read_enable = '1' THEN
                    IF Cas_latency_3 = '1' THEN
                        IF Addr(10) = '1' THEN
                            Command(6) := READ_A;
                        ELSE
                            Command(6) := READ;
                        END IF;
                        Col_addr(6) := TO_BITVECTOR (Addr(col_bits - 1 DOWNTO 0));
                        Bank_addr(6) := TO_BITVECTOR (Ba);
                    ELSIF Cas_latency_25 = '1' THEN
                        IF Addr(10) = '1' THEN
                            Command(5) := READ_A;
                        ELSE
                            Command(5) := READ;
                        END IF;
                        Col_addr(5) := TO_BITVECTOR (Addr(col_bits - 1 DOWNTO 0));
                        Bank_addr(5) := TO_BITVECTOR (Ba);
                    ELSIF Cas_latency_2 = '1' THEN
                        IF Addr(10) = '1' THEN
                            Command(4) := READ_A;
                        ELSE
                            Command(4) := READ;
                        END IF;
                        Col_addr(4) := TO_BITVECTOR (Addr(col_bits - 1 DOWNTO 0));
                        Bank_addr(4) := TO_BITVECTOR (Ba);
                    END IF;

                -- Write Command
                ELSIF Write_enable = '1' THEN
                    IF Addr(10) = '1' THEN
                        Command(2) := WRITE_A;
                    ELSE
                        Command(2) := WRITE;
                    END IF;
                    Col_addr(2) := TO_BITVECTOR (Addr(col_bits - 1 DOWNTO 0));
                    Bank_addr(2) := TO_BITVECTOR (Ba);
                END IF;

                -- Read or Write with Auto Precharge
                IF Addr(10) = '1' THEN
                    Auto_precharge (TO_INTEGER(Ba)) := '1';
                    Count_precharge (TO_INTEGER(Ba)) := 0;
                    IF Read_enable = '1' THEN
                        Read_precharge (TO_INTEGER(Ba)) := '1';
                    ELSIF Write_enable = '1' THEN
                        Write_precharge (TO_INTEGER(Ba)) := '1';
                    END IF;
                END IF;
            END IF;

            -- Read with AutoPrecharge Calculation
            --      The device start internal precharge when:
            --          1.  BL/2 cycles after command
            --          2.  Meet tRAS requirement
            IF ((Auto_precharge(0) = '1') AND (Read_precharge(0) = '1') AND (NOW - RAS_chk0 >= tRAS)) THEN
                IF ((Burst_length_2 = '1' AND Count_precharge(0) >= 1)  OR
                    (Burst_length_4 = '1' AND Count_precharge(0) >= 2)  OR
                    (Burst_length_8 = '1' AND Count_precharge(0) >= 4)) THEN
                    Pc_b0 := '1';
                    Act_b0 := '0';
                    RP_chk0 := NOW;
                    Auto_precharge(0) := '0';
                    Read_precharge(0) := '0';
                END IF;
            END IF;
            IF ((Auto_precharge(1) = '1') AND (Read_precharge(1) = '1') AND (NOW - RAS_chk1 >= tRAS)) THEN
                IF ((Burst_length_2 = '1' AND Count_precharge(1) >= 1)  OR
                    (Burst_length_4 = '1' AND Count_precharge(1) >= 2)  OR
                    (Burst_length_8 = '1' AND Count_precharge(1) >= 4)) THEN
                    Pc_b1 := '1';
                    Act_b1 := '0';
                    RP_chk1 := NOW;
                    Auto_precharge(1) := '0';
                    Read_precharge(1) := '0';
                END IF;
            END IF;
            IF ((Auto_precharge(2) = '1') AND (Read_precharge(2) = '1') AND (NOW - RAS_chk2 >= tRAS)) THEN
                IF ((Burst_length_2 = '1' AND Count_precharge(2) >= 1)  OR
                    (Burst_length_4 = '1' AND Count_precharge(2) >= 2)  OR
                    (Burst_length_8 = '1' AND Count_precharge(2) >= 4)) THEN
                    Pc_b2 := '1';
                    Act_b2 := '0';
                    RP_chk2 := NOW;
                    Auto_precharge(2) := '0';
                    Read_precharge(2) := '0';
                END IF;
            END IF;
            IF ((Auto_precharge(3) = '1') AND (Read_precharge(3) = '1') AND (NOW - RAS_chk3 >= tRAS)) THEN
                IF ((Burst_length_2 = '1' AND Count_precharge(3) >= 1)  OR
                    (Burst_length_4 = '1' AND Count_precharge(3) >= 2)  OR
                    (Burst_length_8 = '1' AND Count_precharge(3) >= 4)) THEN
                    Pc_b3 := '1';
                    Act_b3 := '0';
                    RP_chk3 := NOW;
                    Auto_precharge(3) := '0';
                    Read_precharge(3) := '0';
                END IF;
            END IF;

            -- Write with AutoPrecharge Calculation
            --      The device start internal precharge when:
            --          1.  tWR cycles after command
            --          2.  Meet tRAS requirement
            IF ((Auto_precharge(0) = '1') AND (Write_precharge(0) = '1') AND (NOW - RAS_chk0 >= tRAS)) THEN
                IF ((Burst_length_2 = '1' AND Count_precharge(0) >= 4)  OR
                    (Burst_length_4 = '1' AND Count_precharge(0) >= 5)  OR
                    (Burst_length_8 = '1' AND Count_precharge(0) >= 7)) THEN
                    Pc_b0 := '1';
                    Act_b0 := '0';
                    RP_chk0 := NOW;
                    Auto_precharge(0) := '0';
                    Write_precharge(0) := '0';
                END IF;
            END IF;
            IF ((Auto_precharge(1) = '1') AND (Write_precharge(1) = '1') AND (NOW - RAS_chk1 >= tRAS)) THEN
                IF ((Burst_length_2 = '1' AND Count_precharge(1) >= 4)  OR
                    (Burst_length_4 = '1' AND Count_precharge(1) >= 5)  OR
                    (Burst_length_8 = '1' AND Count_precharge(1) >= 7)) THEN
                    Pc_b1 := '1';
                    Act_b1 := '0';
                    RP_chk1 := NOW;
                    Auto_precharge(1) := '0';
                    Write_precharge(1) := '0';
                END IF;
            END IF;
            IF ((Auto_precharge(2) = '1') AND (Write_precharge(2) = '1') AND (NOW - RAS_chk2 >= tRAS)) THEN
                IF ((Burst_length_2 = '1' AND Count_precharge(2) >= 4)  OR
                    (Burst_length_4 = '1' AND Count_precharge(2) >= 5)  OR
                    (Burst_length_8 = '1' AND Count_precharge(2) >= 7)) THEN
                    Pc_b2 := '1';
                    Act_b2 := '0';
                    RP_chk2 := NOW;
                    Auto_precharge(2) := '0';
                    Write_precharge(2) := '0';
                END IF;
            END IF;
            IF ((Auto_precharge(3) = '1') AND (Write_precharge(3) = '1') AND (NOW - RAS_chk3 >= tRAS)) THEN
                IF ((Burst_length_2 = '1' AND Count_precharge(3) >= 4)  OR
                    (Burst_length_4 = '1' AND Count_precharge(3) >= 5)  OR
                    (Burst_length_8 = '1' AND Count_precharge(3) >= 7)) THEN
                    Pc_b3 := '1';
                    Act_b3 := '0';
                    RP_chk3 := NOW;
                    Auto_precharge(3) := '0';
                    Write_precharge(3) := '0';
                END IF;
            END IF;
        END IF;

        -- Internal Precharge or Bst
        IF Command(0) = PRECH THEN      -- Precharge terminate current read if same bank or all bank
            IF Bank_precharge(0) = Bank OR A10_precharge(0) = '1' THEN
                IF Data_out_enable = '1' THEN
                    Dqs_out <= "ZZ";
                    Data_out_enable := '0';
                END IF;
            END IF;
        ELSIF Command(0) = BST THEN     -- Burst terminate current read
            IF Data_out_enable = '1' THEN
                Dqs_out <= "ZZ";
                Data_out_enable := '0';
            END IF;
        END IF;

        WAIT FOR 1 ps;                  -- Delay for Dqs to go HiZ then accept new internal Dqs

        -- Dqs Generator
        IF Data_out_enable = '1' THEN
            Dqs_int <= "00";
            IF Dqs_out = "00" THEN
                Dqs_out <= "11";
            ELSIF Dqs_out = "11" THEN
                Dqs_out <= "00";
            ELSE
                Dqs_out <= "00";
            END IF;
        ELSIF Data_out_enable = '0' AND Dqs_int = "00" THEN
            Dq <= (OTHERS => 'Z');
            Dqs_out <= "ZZ";
        END IF;

        WAIT FOR 1 ps;                  -- Delay for Dqs to go HiZ then accept new Read or Write

        -- Internal Dqs
        IF Command(2) = READ OR Command(2) = READ_A THEN
            IF (Data_out_enable = '0') THEN
                Dqs_out <= "00";
                Dqs_int <= "11";
            END IF;
        ELSIF (Command(2) = WRITE OR Command(2) = WRITE_A) THEN
            Dqs_out <= "ZZ";
            Dqs_int <= "00";
        END IF;

        -- Detect Read or Write Command
        Rw_command <= Command(1);

        -- Checking internal wires
        Pre_chk (0) <= Pc_b0;
        Pre_chk (1) <= Pc_b1;
        Pre_chk (2) <= Pc_b2;
        Pre_chk (3) <= Pc_b3;
        Act_chk (0) <= Act_b0;
        Act_chk (1) <= Act_b1;
        Act_chk (2) <= Act_b2;
        Act_chk (3) <= Act_b3;
    END PROCESS;

    -- Read or Write Command waiting for Dqs
    rw_decode : PROCESS
        -- Latch Address for Read or Write
        PROCEDURE Latch_address IS
        BEGIN
            Bank_dqs <= Bank_addr (1);
            Col_dqs <= Col_addr (1);
            Col_brst_dqs <= Col_addr (1);
            IF Bank_addr (1) = "00" THEN
                Row_dqs <= B0_row_addr;
            ELSIF Bank_addr (1) = "01" THEN
                Row_dqs <= B1_row_addr;
            ELSIF Bank_addr (1) = "10" THEN
                Row_dqs <= B2_row_addr;
            ELSIF Bank_addr (1) = "11" THEN
                Row_dqs <= B3_row_addr;
            END IF;
        END;

    BEGIN
        WAIT ON Rw_command;
        IF (Rw_command = READ  OR Rw_command = READ_A) THEN
            Latch_address;
            WAIT ON Sys_clk;
            Bank <= Bank_dqs;
            Row  <= Row_dqs;
            Col  := Col_dqs;
            Col_brst <= Col_brst_dqs;
            Burst_counter := 0;
            Data_in_enable := '0';
            Data_out_enable := '1';
        ELSIF (Rw_command = WRITE OR Rw_command = WRITE_A) THEN
            Latch_address;
            WAIT ON Dqs UNTIL Dqs = "11";
            Bank <= Bank_dqs;
            Row  <= Row_dqs;
            Col  := Col_dqs;
            Col_brst <= Col_brst_dqs;
            Burst_counter := 0;
            Data_in_enable := '1';
            Data_out_enable := '0';
        END IF;
    END PROCESS;

    -- Dqs Buffer (Driver / Receiver)
    Dqs_buffer : PROCESS
        TYPE ram_type IS ARRAY (2**col_bits - 1 DOWNTO 0) OF BIT_VECTOR (data_bits - 1 DOWNTO 0);
        TYPE ram_pntr IS ACCESS ram_type;
        TYPE ram_stor IS ARRAY (2**addr_bits - 1 DOWNTO 0) OF ram_pntr;
        VARIABLE Bank0 : ram_stor;
        VARIABLE Bank1 : ram_stor;
        VARIABLE Bank2 : ram_stor;
        VARIABLE Bank3 : ram_stor;
        VARIABLE Row_index, Col_index : INTEGER := 0;

        -- Initialize empty rows
        PROCEDURE Init_mem (Bank : BIT_VECTOR; Row_index : INTEGER) IS
            VARIABLE i, j : INTEGER := 0;
        BEGIN
            IF Bank = "00" THEN
                IF Bank0 (Row_index) = NULL THEN                        -- Check to see if row empty
                    Bank0 (Row_index) := NEW ram_type;                  -- Open new row for access
                    FOR i IN (2**col_bits - 1) DOWNTO 0 LOOP            -- Filled row with zeros
                        FOR j IN (data_bits - 1) DOWNTO 0 LOOP
                            Bank0 (Row_index) (i) (j) := '0';
                        END LOOP;
                    END LOOP;
                END IF;
            ELSIF Bank = "01" THEN
                IF Bank1 (Row_index) = NULL THEN
                    Bank1 (Row_index) := NEW ram_type;
                    FOR i IN (2**col_bits - 1) DOWNTO 0 LOOP
                        FOR j IN (data_bits - 1) DOWNTO 0 LOOP
                            Bank1 (Row_index) (i) (j) := '0';
                        END LOOP;
                    END LOOP;
                END IF;
            ELSIF Bank = "10" THEN
                IF Bank2 (Row_index) = NULL THEN
                    Bank2 (Row_index) := NEW ram_type;
                    FOR i IN (2**col_bits - 1) DOWNTO 0 LOOP
                        FOR j IN (data_bits - 1) DOWNTO 0 LOOP
                            Bank2 (Row_index) (i) (j) := '0';
                        END LOOP;
                    END LOOP;
                END IF;
            ELSIF Bank = "11" THEN
                IF Bank3 (Row_index) = NULL THEN
                    Bank3 (Row_index) := NEW ram_type;
                    FOR i IN (2**col_bits - 1) DOWNTO 0 LOOP
                        FOR j IN (data_bits - 1) DOWNTO 0 LOOP
                            Bank3 (Row_index) (i) (j) := '0';
                        END LOOP;
                    END LOOP;
                END IF;
            END IF;
        END;
            
        -- Burst Counter
        PROCEDURE Burst_decode IS
            VARIABLE Col_int : INTEGER := 0;
            VARIABLE Col_temp, Col_vec : BIT_VECTOR (col_bits - 1 DOWNTO 0) := (OTHERS => '0');
        BEGIN
            -- Advance burst counter
            Burst_counter := Burst_counter + 1;

            -- Burst Type
            IF Mode_reg (3) = '0' THEN
                Col_int := TO_INTEGER(Col);
                Col_int := Col_int + 1;
                TO_BITVECTOR (Col_int, Col_temp);
            ELSIF Mode_reg (3) = '1' THEN
                TO_BITVECTOR (Burst_counter, Col_vec);
                Col_temp (2) := Col_vec (2) XOR Col_brst (2);
                Col_temp (1) := Col_vec (1) XOR Col_brst (1);
                Col_temp (0) := Col_vec (0) XOR Col_brst (0);
            END IF;

            -- Burst Length
            IF Burst_length_2 = '1' THEN
                Col(0) := Col_temp (0);
            ELSIF Burst_length_4 = '1' THEN
                Col(1 DOWNTO 0) := Col_temp (1 DOWNTO 0);
            ELSIF Burst_length_8 = '1' THEN
                Col(2 DOWNTO 0) := Col_temp (2 DOWNTO 0);
            ELSE
                Col := Col_temp;
            END IF;

            -- Data counter
            IF Burst_length_2 = '1' THEN
                IF Burst_counter >= 2 THEN
                    IF Data_in_enable = '1' THEN
                        Data_in_enable := '0';
                    ELSIF Data_out_enable = '1' THEN
                        Data_out_enable := '0';
                    END IF;
                END IF;
            ELSIF Burst_length_4 = '1' THEN
                IF Burst_counter >= 4 THEN
                    IF Data_in_enable = '1' THEN
                        Data_in_enable := '0';
                    ELSIF Data_out_enable = '1' THEN
                        Data_out_enable := '0';
                    END IF;
                END IF;
            ELSIF Burst_length_8 = '1' THEN
                IF Burst_counter >= 8 THEN
                    IF Data_in_enable = '1' THEN
                        Data_in_enable := '0';
                    ELSIF Data_out_enable = '1' THEN
                        Data_out_enable := '0';
                    END IF;
                END IF;
            END IF;
        END;

    BEGIN
        WAIT ON Dqs;
        WAIT FOR 1 ps;                                                  -- Delay to avoid race condition with Rw_command
        Row_index := TO_INTEGER (Row);
        Col_index := TO_INTEGER (Col);
        IF Data_in_enable = '1' THEN
            IF Dm = "00" THEN
                Init_mem (Bank, Row_index);
                IF Bank = "00" THEN
                    Bank0 (Row_index) (Col_index) := TO_BITVECTOR (Dq);
                ELSIF Bank = "01" THEN
                    Bank1 (Row_index) (Col_index) := TO_BITVECTOR (Dq);
                ELSIF Bank = "10" THEN
                    Bank2 (Row_index) (Col_index) := TO_BITVECTOR (Dq);
                ELSIF Bank = "11" THEN
                    Bank3 (Row_index) (Col_index) := TO_BITVECTOR (Dq);
                END IF;
                WR_chk (TO_INTEGER(Bank)) := 0;
            END IF;
            Burst_decode;
        ELSIF Data_out_enable = '1' THEN    
            Init_mem (Bank, Row_index);
            IF Bank = "00" THEN
                Dq <= TO_STDLOGICVECTOR (Bank0 (Row_index) (Col_index));
            ELSIF Bank = "01" THEN
                Dq <= TO_STDLOGICVECTOR (Bank1 (Row_index) (Col_index));
            ELSIF Bank = "10" THEN
                Dq <= TO_STDLOGICVECTOR (Bank2 (Row_index) (Col_index));
            ELSIF Bank = "11" THEN
                Dq <= TO_STDLOGICVECTOR (Bank3 (Row_index) (Col_index));
            END IF;
            Burst_decode;
        ELSE
            Dq <= (OTHERS => 'Z');
        END IF;
    END PROCESS;

    -- Setup timing checks
    Setup_check : PROCESS
    BEGIN
        WAIT ON Sys_clk;
        IF Sys_clk = '1' THEN
            ASSERT(Cke'LAST_EVENT >= tIS)
                REPORT "CKE Setup time violation -- tIS"
                SEVERITY WARNING;
            ASSERT(Cs_n'LAST_EVENT >= tIS)
                REPORT "CS# Setup time violation -- tIS"
                SEVERITY WARNING;
            ASSERT(Cas_n'LAST_EVENT >= tIS)
                REPORT "CAS# Setup time violation -- tIS"
                SEVERITY WARNING;
            ASSERT(Ras_n'LAST_EVENT >= tIS)
                REPORT "RAS# Setup time violation -- tIS"
                SEVERITY WARNING;
            ASSERT(We_n'LAST_EVENT >= tIS)
                REPORT "WE# Setup time violation -- tIS"
                SEVERITY WARNING;
            ASSERT(Addr'LAST_EVENT >= tIS)
                REPORT "ADDR Setup time violation -- tIS"
                SEVERITY WARNING;
            ASSERT(Ba'LAST_EVENT >= tIS)
                REPORT "BA Setup time violation -- tIS"
                SEVERITY WARNING;
        END IF;   
    END PROCESS;

    -- Hold timing checks
    Hold_check : PROCESS
    BEGIN
        WAIT ON Sys_clk'DELAYED (tIH);
        IF Sys_clk'DELAYED (tIH) = '1' THEN
            ASSERT(Cke'LAST_EVENT >= tIH)
                REPORT "CKE Setup time violation -- tIH"
                SEVERITY WARNING;
            ASSERT(Cs_n'LAST_EVENT >= tIH)
                REPORT "CS# Setup time violation -- tIH"
                SEVERITY WARNING;
            ASSERT(Cas_n'LAST_EVENT >= tIH)
                REPORT "CAS# Setup time violation -- tIH"
                SEVERITY WARNING;
            ASSERT(Ras_n'LAST_EVENT >= tIH)
                REPORT "RAS# Setup time violation -- tIH"
                SEVERITY WARNING;
            ASSERT(We_n'LAST_EVENT >= tIH)
                REPORT "WE# Setup time violation -- tIH"
                SEVERITY WARNING;
            ASSERT(Addr'LAST_EVENT >= tIH)
                REPORT "ADDR Setup time violation -- tIH"
                SEVERITY WARNING;
            ASSERT(Ba'LAST_EVENT >= tIH)
                REPORT "BA Setup time violation -- tIH"
                SEVERITY WARNING;
        END IF;   
    END PROCESS;

END behave;
