/******************************************************************************
 *
 *    File Name:  brst_cntr.v
 *      Version:  2.5
 *         Date:  June 05, 2000
 * 
 *  Description:  generate burst_end 
 *                For burst of 2 & 4, controller never goes to 
 *                   READ_DATA & WRITE_DATA states
 *                For burst of 8: controller stays in 
 *                   READ_DATA or WRITE_DAT untill burst_end is true
 *                burst_end follows ld_burst by 2 clk cycle
 *
 *       Author:  Jennnifer Tran
 *      Company:  Xilinx
 *
 *                Copyright (c) 1999 Xilinx, Inc.
 *                All rights reserved
 *
 *    DDR SDRAM:  64M (1M x 16Bit x 4Bank)
 * 
 *   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
 *                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
 *                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
 *                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
 *
 ******************************************************************************/
   
`timescale  100 ps / 10 ps

module brst_cntr ( /*AUTOARG*/
   // Outputs
   brst_end, 
   // Inputs
   clk, ld_brst
   );
   
   input clk, ld_brst;
   output brst_end;

   reg 	  b_1, brst_end;
   
   always @ (posedge clk) 
   begin
      b_1 <= ~ld_brst;
      brst_end <= b_1;
   end
   
endmodule // brst_cntr
