/******************************************************************************
 *
 *    File Name:  cslt_cntr.v
 *      Version:  2.5
 *         Date:  June 05, 2000
 * 
 *  Description:  counts from cslt_max down to 0.  When counter = 0, cslt_end goes high
 *
 *       Author:  Jennnifer Tran
 *      Company:  Xilinx
 *
 *                Copyright (c) 1999 Xilinx, Inc.
 *                All rights reserved
 *
 *    DDR SDRAM:  64M (1M x 16Bit x 4Bank)
 * 
 *   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
 *                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
 *                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
 *                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
 *
 ******************************************************************************/

`timescale  100 ps / 10 ps

module cslt_cntr (   cslt_end, 
   cslt_max, clk, clk2x, ld_cslt, cas_lat_half
   );
   
   input [1:0] cslt_max;
   input       clk, clk2x, ld_cslt, cas_lat_half;
   output      cslt_end;

   reg [1:0]   count;
   reg 	       cslt_end1_5;  //for CAS lat of 1.5 and 2.5, need to delay cslt_end by 1/2 a clk cycle
   
   wire        cslt_end1 = ~(|count); //cslt_end goes high when count=0
   reg 	       cslt_end;
   
   always @ (posedge clk)
     cslt_end <= cas_lat_half ? cslt_end1 : cslt_end1_5;
   
   always @ (posedge clk or posedge ld_cslt)
     if (ld_cslt)
       count <= (cslt_max);
     else
	count <= count - 1;

   always @ (posedge clk2x)
     cslt_end1_5 <= cslt_end1;

endmodule 

