library IEEE;
use IEEE.STD_LOGIC_1164.all;
library work;
--use WORK.ddr_controller_comp.all;
-- synopsys translate_off
--library unisim;
--use unisim.vcomponents.all;
-- synopsys translate_on

package ddr_controller_comp is

  constant DDR_ADDR_MSB      :integer := 11;
  constant DDR_DATA_MSB      :integer := 63;
  constant SYS_ADDR_MSB      :integer := 21;
  constant SYS_DATA_MSB      :integer := 15; 
  constant U_ADDR_MSB        :integer := 21;
  constant U_DATA_MSB        :integer := 127;
  constant ENABLE_MSB        :integer := 15;
  -- system commands
  constant SYS_NOP           :integer :=1;
  constant SYS_LOAD_MR       :integer :=2;
  constant SYS_READ          :integer :=3;
  constant SYS_WRITE         :integer :=4;
  constant SYS_PRECHARGE     :integer :=5;
  constant SYS_REFRESH       :integer :=6;
  constant SYS_BURST_STOP    :integer :=7;

  constant cmd_SYS_NOP           :std_logic_vector :="0000001";
  constant cmd_SYS_LOAD_MR       :std_logic_vector :="0000010";
  constant cmd_SYS_READ          :std_logic_vector :="0000100";
  constant cmd_SYS_WRITE         :std_logic_vector :="0001000";
  constant cmd_SYS_PRECHARGE     :std_logic_vector :="0010000";
  constant cmd_SYS_REFRESH       :std_logic_vector :="0100000";
  constant cmd_SYS_BURST_STOP    :std_logic_vector :="1000000";

  --ddr controller states
  constant CTLR_IDLE         :integer :=1;
  constant CTLR_REFRESH      :integer :=2;
  constant CTLR_PRECHARGE    :integer :=3;
  constant CTLR_LOAD_MR      :integer :=4;
  constant CTLR_ACT          :integer :=5;
  constant CTLR_ACT_WAIT     :integer :=6;
  constant CTLR_READ         :integer :=7;
  constant CTLR_WRITE        :integer :=8;
  constant CTLR_BURST_STOP   :integer :=9;
  constant CTLR_READ_WAIT    :integer :=10;
  constant CTLR_READ_DATA    :integer :=11;
  constant CTLR_WRITE_DATA   :integer :=12;

  component addr_latch
    port(
    --Inputs
    u_addr	  : in std_logic_vector(U_ADDR_MSB downto 0);
    row_addr	  : in std_logic;
    mrs_addr	  : in std_logic;
    clk           : in std_logic;
    reset1        : in std_logic;
    reset2        : in std_logic;    
    --Outputs
    burst_8	  : out std_logic;
    burst_2	  : out std_logic;
    cas_lat_max   : out std_logic_vector(1 downto 0);
    cas_lat_half  : out std_logic_vector(1 downto 0);
    ddr_ad	  : out std_logic_vector(DDR_ADDR_MSB downto 0);
    ddr_ba	  : out std_logic_vector(1 downto 0));
  end component;
  
  component brst_cntr
    port(
      --Inputs
      clk       : in std_logic;
      ld_brst   : in std_logic;
      --Output
      brst_end  : out std_logic);
  end component;

  component clk_dlls
    port(
      --Inputs
      sys_clk_in         : in std_logic;
      sys_clk_fb_in      : in std_logic;
      reset_in           : in std_logic;
      --Outputs
      ddr_clk_out        : out std_logic;
      ddr_clk180_out     : out std_logic;
      fpga_clk2x_out     : out std_logic;
      fpga_clk_out       : out std_logic;
      dlls_locked_out    : out std_logic;
      fpga_lac_clk_out   : out std_logic;
      fpga_lac_clk90_out : out std_logic);
  end component;

  component controller
    port(
    --Outputs
    ddr_rasb		: out std_logic;
    ddr_casb		: out std_logic;
    ddr_web		: out std_logic;
    ld_burst            : out std_logic;
    ld_rcd		: out std_logic;
    ld_cas_lat		: out std_logic;
    u_ref_ack		: out std_logic;
    ddr_read_en		: out std_logic_vector(ENABLE_MSB downto 0);
    ddr_write_en	: out std_logic_vector(ENABLE_MSB downto 0);
    ddr_dqs_toggle	: out std_logic;
    u_data_valid_en	: out std_logic;
    ddr_dqs_t           : out std_logic;
    mrs_addr		: out std_logic;
    row_addr		: out std_logic;
    --Inputs
    u_cmd		: in std_logic_vector(7 downto 1);
    cntlr_reset1        : in std_logic;
    cntlr_reset2        : in std_logic;
    cntlr_reset3        : in std_logic;
    clk                 : in std_logic;
    clk2x		: in std_logic;
    lac_clk_in		: in std_logic;
    burst_end		: in std_logic;
    cas_lat_end		: in std_logic;
    rcd_end		: in std_logic;
    burst_8		: in std_logic;
    burst_2		: in std_logic);
  end component;

  component cslt_cntr
    port(
      --Inputs
      cslt_max     : in std_logic_vector(1 downto 0);
      clk          : in std_logic;
      clk2x        : in std_logic;
      ld_cslt      : in std_logic;
      cas_lat_half : in std_logic;
      --Output
      cslt_end     : out std_logic);
  end component;

  component data_path
    port(
    --Outputs
    u_data_o	    : out std_logic_vector(U_DATA_MSB downto 0);
    ddr_dqs	    : out std_logic_vector(1 downto 0);
    u_data_valid    : out std_logic;
    
    --Inout
    ddr_dq	    : inout std_logic_vector(DDR_DATA_MSB downto 0);
    
    --Inputs
    u_data_i	    : in std_logic_vector(U_DATA_MSB downto 0);
    ddr_write_en    : in std_logic_vector(ENABLE_MSB downto 0);
    ddr_dqs_t	    : in std_logic;
    u_data_valid_en : in std_logic;
    cas_lat_half    : in std_logic_vector(1 downto 0);
    burst_8         : in std_logic;        
    ddr_read_en     : in std_logic_vector(ENABLE_MSB downto 0);
    clk2x	    : in std_logic;
    clk             : in std_logic;
    lac_clk_in	    : in std_logic);
  end component;

  component ddr_ctlr
    port(
    --Outputs
    ddr_ad	: out std_logic_vector(DDR_ADDR_MSB downto 0);
    ddr_dm	: out std_logic_vector(1 downto 0);
    ddr_ba	: out std_logic_vector(1 downto 0);
    ddr_rasb	: out std_logic;
    ddr_casb	: out std_logic;
    ddr_web	: out std_logic;
    ddr_clk	: out std_logic;
    ddr_clkb	: out std_logic;
    ddr_dqs	: out std_logic_vector(1 downto 0);
    ddr_csb	: out std_logic;
    ddr_cke	: out std_logic;
    u_data_o	: out std_logic_vector(U_DATA_MSB downto 0);
    u_ref_ack	: out std_logic;
    u_data_valid: out std_logic;
    fpga_clk	: out std_logic;

    --Inout
    ddr_dq	: inout std_logic_vector(DDR_DATA_MSB downto 0);
    
    --Inputs
    u_addr	: in std_logic_vector(U_ADDR_MSB downto 0);
    u_data_i	: in std_logic_vector(U_DATA_MSB downto 0);
    u_cmd	: in std_logic_vector(7 downto 1);
    u_reset_n	: in std_logic;
    u_clk	: in std_logic;
    u_clk_fb	: in std_logic);
  end component;

  
  component rcd_cntr
    port(
    --Outputs
    rcd_end : out std_logic;
    
    --Inputs
    clk     : in std_logic;
    ld_rcd  : in std_logic);
  end component;

  component user_int
    port(
      --Outputs
      sys_data_o     : out std_logic_vector(SYS_DATA_MSB downto 0);
      sys_ref_ack    : out std_logic;
      sys_data_valid : out std_logic;
      u_addr	     : out std_logic_vector(U_ADDR_MSB downto 0);
      u_data_i	     : out std_logic_vector(U_DATA_MSB downto 0);
      u_cmd	     : out std_logic_vector(7 downto 1);
      u_reset_n	     : out std_logic;
      u_clk	     : out std_logic;
      u_clk_fb       : out std_logic;
      --Inputs
      sys_addr	     :in std_logic_vector(U_ADDR_MSB downto 0);
      sys_data_i     :in std_logic_vector(SYS_DATA_MSB downto 0);
      sys_cmd	     :in std_logic_vector(7 downto 1);
      sys_reset_n    :in std_logic;
      sys_clk	     :in std_logic;
      sys_clk_fb     :in std_logic;
      u_data_o	     :in std_logic_vector(U_DATA_MSB downto 0);
      u_ref_ack	     :in std_logic;
      u_data_valid   :in std_logic;
      fpga_clk	     :in std_logic);
  end component;

  component top
    port(
      --Outputs
      ddr_ad	     : out std_logic_vector(DDR_ADDR_MSB downto 0);
      ddr_dm	     : out std_logic_vector(1 downto 0);
      ddr_ba	     : out std_logic_vector(1 downto 0);
      ddr_rasb	     : out std_logic;
      ddr_casb       : out std_logic;
      ddr_web        : out std_logic;
      ddr_clk	     : out std_logic;
      ddr_clkb	     : out std_logic;
      ddr_dqs	     : out std_logic_vector(1 downto 0);
      ddr_csb	     : out std_logic;
      ddr_cke	     : out std_logic;
      sys_data_o     : out std_logic_vector(SYS_DATA_MSB downto 0);
      sys_ref_ack    : out std_logic;
      sys_data_valid : out std_logic;
      --Inputs
      sys_addr	     : in std_logic_vector(U_ADDR_MSB downto 0);
      sys_data_i     : in std_logic_vector(SYS_DATA_MSB downto 0);
      sys_cmd	     : in std_logic_vector(7 downto 1);
      sys_reset_n    : in std_logic;
      sys_clk	     : in std_logic;
      sys_clk_fb     : in std_logic;
      --Inouts
      ddr_dq	     : inout std_logic_vector(DDR_DATA_MSB downto 0));
  end component;

-------------------------------------------------------------------------------
-- components from Xilinx library ---------------------------------------------
-------------------------------------------------------------------------------
  component CLKDLL
      port ( CLKIN   : in  std_ulogic := '0';
           CLKFB   : in  std_ulogic := '0';
           RST     : in  std_ulogic := '0';
           CLK0    : out std_ulogic := '0';
           CLK90   : out std_ulogic := '0';
           CLK180  : out std_ulogic := '0';
           CLK270  : out std_ulogic := '0';
           CLK2X   : out std_ulogic := '0';
           CLKDV   : out std_ulogic := '0';
           LOCKED  : out std_ulogic := '0');
  end component;
  component SRL16
    port(Q : out std_ulogic;
         A0: in  std_ulogic;
         A1: in  std_ulogic;
         A2: in  std_ulogic;
         A3: in  std_ulogic;
         D : in  std_ulogic;
         CLK: in std_ulogic);
  end component;

  component SRL16E
    port(D  : in  std_ulogic;
         CLK: in  std_ulogic;
         CE : in  std_ulogic;
         A0 : in  std_ulogic;
         A1 : in  std_ulogic;
         A2 : in  std_ulogic;
         A3 : in  std_ulogic;
         Q  : out std_ulogic);
  end component;

  component IBUFG_SSTL2_I
    port(I : in std_logic;
         O : out std_logic);
  end component;

  component IBUFG
    port(I : in std_logic;
         O : out std_logic);
  end component;

  component IBUF_SSTL2_I
    port(I : in std_logic;
         O : out std_logic);
  end component;

  component OBUF_SSTL2_I
    port(I : in std_logic;
         O : out std_logic);
  end component;
  
  component OBUFT_SSTL2_II
    port(I : in std_logic;
         T : in std_logic;
         O : out std_logic);
  end component;
  
  component IOBUF_SSTL2_II
    port(I : in std_logic;
         IO: inout std_logic;
         T : in std_logic;
         O : out std_logic);
  end component;

  component BUFG
    port(I : in std_logic;
         O : out std_logic);
  end component;

  component FD
    port(D  : in std_logic;
         C  : in std_logic;
         Q  : out std_logic);
  end component;

  component FDP
    port(D  : in std_logic;
         C  : in std_logic;
         PRE: in std_logic;
         Q  : out std_logic);
  end component;
end ddr_controller_comp;

