--*****************************************************************************
--
--    File Name:  ddr_ctlr.vhd
--      Version:  2.5
--         Date:  June 05, 2000
-- 
--  Description:  controller for DDR SDRAM  
-- 
--       Author:  Jennnifer Tran
--      Company:  Xilinx
--
--                Copyright (c) 1999 Xilinx, Inc.
--                All rights reserved
--
--    DDR SDRAM:  64M (1M x 16Bit x 4Bank)
-- 
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--*****************************************************************************/

library IEEE;
use IEEE.std_logic_1164.all;
library work;
use WORK.ddr_controller_comp.all;
-- synopsys translate_off
library unisim;
use unisim.vcomponents.all;
-- synopsys translate_on

entity ddr_ctlr is
  port(
    --Outputs
    ddr_ad	: out std_logic_vector(DDR_ADDR_MSB downto 0);
    ddr_dm	: out std_logic_vector(1 downto 0);
    ddr_ba	: out std_logic_vector(1 downto 0);
    ddr_rasb	: out std_logic;
    ddr_casb	: out std_logic;
    ddr_web	: out std_logic;
    ddr_clk	: out std_logic;
    ddr_clkb	: out std_logic;
    ddr_dqs	: out std_logic_vector(1 downto 0);
    ddr_csb	: out std_logic;
    ddr_cke	: out std_logic;
    u_data_o	: out std_logic_vector(U_DATA_MSB downto 0);
    u_ref_ack	: out std_logic;
    u_data_valid: out std_logic;
    fpga_clk	: out std_logic;

    --Inout
    ddr_dq	: inout std_logic_vector(DDR_DATA_MSB downto 0);
    
    --Inputs
    u_addr	: in std_logic_vector(U_ADDR_MSB downto 0);
    u_data_i	: in std_logic_vector(U_DATA_MSB downto 0);
    u_cmd	: in std_logic_vector(7 downto 1);
    u_reset_n	: in std_logic;
    u_clk	: in std_logic;
    u_clk_fb	: in std_logic);
end ddr_ctlr;

architecture rtl of ddr_ctlr is
  signal ddr_burst_length : std_logic_vector(1 downto 0);
  signal ddr_cas_latency  : std_logic_vector(2 downto 0);
  signal cas_lat_max	  : std_logic_vector(1 downto 0);
  signal dlls_locked	  : std_logic;
  signal reset		  : std_logic;
  signal ddr_write_en	  : std_logic_vector(ENABLE_MSB downto 0);
  signal ddr_read_en	  : std_logic_vector(ENABLE_MSB downto 0);
  signal ctlr_reset1	  : std_logic;
  signal ctlr_reset2	  : std_logic;
  signal ctlr_reset3	  : std_logic;
  signal addr_reset1	  : std_logic;
  signal addr_reset2	  : std_logic;
  signal fpga_clk_a	  : std_logic;
  signal logic0,logic1	  : std_logic;
  signal fpga_clk2x	  : std_logic;
  signal fpga_lac_clk	  : std_logic;
  signal fpga_lac_clk90	  : std_logic;
  signal ddr_dqs_t	  : std_logic;
  signal row_addr	  : std_logic;
  signal mrs_addr	  : std_logic;
  signal addr_latch_en    : std_logic;
  signal ld_burst	  : std_logic;
  signal ld_rcd		  : std_logic;
  signal ld_cas_lat	  : std_logic;
  signal burst_end	  : std_logic;
  signal cas_lat_end	  : std_logic;
  signal rcd_end	  : std_logic;
  signal ld_cslt	  : std_logic;
  signal burst_8          : std_logic;
  signal burst_2          : std_logic;
  signal cas_lat_half     : std_logic_vector(1 downto 0);
  signal u_data_valid_en  : std_logic;
  
  component addr_latch
    port(
      --Inputs
    u_addr	  : in std_logic_vector(U_ADDR_MSB downto 0);
    row_addr	  : in std_logic;
    mrs_addr	  : in std_logic;
    clk           : in std_logic;
    reset1        : in std_logic;
    reset2        : in std_logic;
    --Outputs
    burst_8	  : out std_logic;
    burst_2	  : out std_logic;
    cas_lat_max   : out std_logic_vector(1 downto 0);
    cas_lat_half  : out std_logic_vector(1 downto 0);
    ddr_ad	  : out std_logic_vector(DDR_ADDR_MSB downto 0);
    ddr_ba	  : out std_logic_vector(1 downto 0));
  end component;
  
  component brst_cntr
    port(
      --Inputs
      clk       : in std_logic;
      ld_brst   : in std_logic;
      --Output
      brst_end  : out std_logic);
  end component;

  component clk_dlls
    port(
      --Inputs
      sys_clk_in         : in std_logic;
      sys_clk_fb_in      : in std_logic;
      reset_in           : in std_logic;
      --Outputs
      ddr_clk_out        : out std_logic;
      ddr_clk180_out     : out std_logic;
      fpga_clk2x_out     : out std_logic;
      fpga_clk_out       : out std_logic;
      dlls_locked_out    : out std_logic;
      fpga_lac_clk_out   : out std_logic;
      fpga_lac_clk90_out : out std_logic);
  end component;

  component controller
    port(
      --Outputs
    ddr_rasb		: out std_logic;
    ddr_casb		: out std_logic;
    ddr_web		: out std_logic;
    ld_burst            : out std_logic;
    ld_rcd		: out std_logic;
    ld_cas_lat		: out std_logic;
    u_ref_ack		: out std_logic;
    ddr_read_en		: out std_logic_vector(ENABLE_MSB downto 0);
    ddr_write_en	: out std_logic_vector(ENABLE_MSB downto 0);
    u_data_valid_en	: out std_logic;
    ddr_dqs_t           : out std_logic;
    mrs_addr		: out std_logic;
    row_addr		: out std_logic;
    
    --Inputs
    u_cmd		: in std_logic_vector(7 downto 1);
    cntlr_reset1        : in std_logic;
    cntlr_reset2        : in std_logic;
    cntlr_reset3        : in std_logic;
    clk                 : in std_logic;
    clk2x		: in std_logic;
    lac_clk_in		: in std_logic;
    burst_end		: in std_logic;
    cas_lat_end		: in std_logic;
    rcd_end		: in std_logic;
    burst_8		: in std_logic;
    burst_2		: in std_logic);
  end component;

  component cslt_cntr
    port(
      --Inputs
      cslt_max     : in std_logic_vector(1 downto 0);
      clk          : in std_logic;
      clk2x        : in std_logic;
      ld_cslt      : in std_logic;
      cas_lat_half : in std_logic;
      --Output
      cslt_end     : out std_logic);
  end component;

  component data_path
    port(
    --Outputs
    u_data_o	    : out std_logic_vector(U_DATA_MSB downto 0);
    ddr_dqs	    : out std_logic_vector(1 downto 0);
    u_data_valid    : out std_logic;    
    --Inout
    ddr_dq	    : inout std_logic_vector(DDR_DATA_MSB downto 0);    
    --Inputs
    u_data_i	    : in std_logic_vector(U_DATA_MSB downto 0);
    ddr_write_en    : in std_logic_vector(ENABLE_MSB downto 0);
    ddr_dqs_t	    : in std_logic;
    u_data_valid_en : in std_logic;
    cas_lat_half    : in std_logic_vector(1 downto 0);
    burst_8         : in std_logic;        
    ddr_read_en     : in std_logic_vector(ENABLE_MSB downto 0);
    clk2x	    : in std_logic;
    clk             : in std_logic;
    lac_clk_in	    : in std_logic);
  end component;
  component rcd_cntr
    port(
      --Outputs
      rcd_end : out std_logic;      
      --Inputs
      clk     : in std_logic;
      ld_rcd  : in std_logic);
  end component;
  component OBUF_SSTL2_I
    port(I : in std_logic;
         O : out std_logic);
  end component;

begin

  reset<= not(dlls_locked and u_reset_n);
  fpga_clk <= fpga_clk_a;
  
  process (fpga_clk_a)
  begin
    if (fpga_clk_a 'event and fpga_clk_a = '1') then
      ctlr_reset1 <= reset;
      ctlr_reset2 <= reset;
      ctlr_reset3 <= reset;
      addr_reset1 <= reset;
      addr_reset2 <= reset;
    end if;
  end process;
  
  u_clk_dlls: clk_dlls
    port map(ddr_clk_out=>ddr_clk, ddr_clk180_out=>ddr_clkb,
             fpga_clk2x_out=>fpga_clk2x, fpga_clk_out=>fpga_clk_a,
             dlls_locked_out=>dlls_locked, sys_clk_in=>u_clk,
             sys_clk_fb_in=>u_clk_fb, reset_in=>logic0, 
             fpga_lac_clk_out=>fpga_lac_clk,
             fpga_lac_clk90_out=>fpga_lac_clk90);

  u_data_path: data_path
    port map(u_data_o=>u_data_o, ddr_dqs=>ddr_dqs,
             u_data_valid=>u_data_valid, burst_8=>burst_8,
             ddr_dq=>ddr_dq, u_data_i=>u_data_i,
             ddr_dqs_t=>ddr_dqs_t, u_data_valid_en=>u_data_valid_en,
             ddr_write_en=>ddr_write_en, ddr_read_en=>ddr_read_en,
             clk2x=>fpga_clk2x, clk=>fpga_clk_a,
             lac_clk_in=>fpga_lac_clk, cas_lat_half=>cas_lat_half);
  
  u_addr_latch: addr_latch
    port map (burst_2=>burst_2, burst_8=>burst_8,
              cas_lat_max=>cas_lat_max, cas_lat_half=>cas_lat_half,
              ddr_ad=>ddr_ad, ddr_ba=>ddr_ba, u_addr=>u_addr,
              row_addr=>row_addr, mrs_addr=>mrs_addr, clk=>fpga_clk_a,
              reset1=>addr_reset1, reset2=>addr_reset2);
  
  u_controller: controller
    port map(ddr_rasb=>ddr_rasb, ddr_casb=>ddr_casb,
             ddr_web=>ddr_web, ld_burst=>ld_burst, ld_rcd=>ld_rcd,
             ld_cas_lat=>ld_cas_lat, u_ref_ack=>u_ref_ack,
             ddr_read_en=>ddr_read_en, u_data_valid_en=>u_data_valid_en,
             ddr_dqs_t=>ddr_dqs_t, ddr_write_en=>ddr_write_en,
             mrs_addr=>mrs_addr, burst_2=>burst_2, burst_8=>burst_8,
             row_addr=>row_addr, u_cmd=>u_cmd, cntlr_reset1=>ctlr_reset1,
             cntlr_reset2=>ctlr_reset2, cntlr_reset3=>ctlr_reset3,
             clk=>fpga_clk_a, clk2x=>fpga_clk2x, lac_clk_in=>fpga_lac_clk90,
             burst_end=>burst_end, cas_lat_end=>cas_lat_end,
             rcd_end=>rcd_end);
  
  u_brst_cntr: brst_cntr
    port map(brst_end=>burst_end,
             clk=>fpga_clk_a, ld_brst=>ld_burst);
  
  u_cslt_cntr: cslt_cntr
    port map(cslt_end=>cas_lat_end, cslt_max=>cas_lat_max,
             cas_lat_half=>cas_lat_half(1), clk=>fpga_clk_a,
             clk2x=>fpga_clk2x, ld_cslt=>ld_cas_lat);
  
  u_rcd_cntr: rcd_cntr
    port map(rcd_end=>rcd_end,
             clk=>fpga_clk_a, ld_rcd=>ld_rcd);
  
  
  logic0 <= '0';
  logic1 <= '1';
  
  --interface to DDR signals
  --all inputs/outputs use SSTL2_II
  O_ddr_dqm0: OBUF_SSTL2_I port map(I=>logic0, O=>ddr_dm(0));
  O_ddr_dqm1: OBUF_SSTL2_I port map(I=>logic0, O=>ddr_dm(1));
  O_ddr_cke:  OBUF_SSTL2_I port map(I=>logic1, O=>ddr_cke);
  O_ddr_csb:  OBUF_SSTL2_I port map(I=>logic0, O=>ddr_csb);

end rtl;
