/******************************************************************************
 *
 *    File Name:  top.v
 *      Version:  2.5
 *         Date:  June 05, 2000
 * 
 *  Description:  instantiate ddr_ctrl and a dummy user_int module.
 *                In order to place and route the ddr_ctrl module, we need to
 *                connect all user interface signals.  The user_int module simply
 *                connects all user signals to IOs.  
 * 
 *                The user should replace user_int module with their own interface.
 * 
 * 
 *       Author:  Jennnifer Tran
 *      Company:  Xilinx
 *
 *                Copyright (c) 1999 Xilinx, Inc.
 *                All rights reserved
 *
 *    DDR SDRAM:  64M (1M x 16Bit x 4Bank)
 * 
 *   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
 *                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
 *                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
 *                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
 *
 * interface to 4 v2m32
 ******************************************************************************/
`timescale 100ps / 10ps
`include "../src/define.v"

module top (/*AUTOARG*/
   // Outputs
   ddr_ad, ddr_dm, ddr_ba, ddr_rasb, ddr_casb, ddr_web, ddr_clk, 
   ddr_clkb, ddr_dqs, ddr_csb, ddr_cke, sys_data_o, sys_ref_ack, 
   sys_data_valid, 
   // Inouts
   ddr_dq, 
   // Inputs
   sys_addr, sys_data_i, sys_cmd, sys_reset_n, sys_clk, sys_clk_fb
   );

   output [`DDR_ADDR_MSB:0] ddr_ad;
   inout [`DDR_DATA_MSB:0]  ddr_dq;   
   output [3:0] 	    ddr_dm;
   output [1:0] 	    ddr_ba; 
   output 		    ddr_rasb, ddr_casb, ddr_web;
   output 		    ddr_clk, ddr_clkb;
   output [3:0] 	    ddr_dqs;
   output 		    ddr_csb, ddr_cke;
   
   input [`SYS_ADDR_MSB:0]  sys_addr;
   input [`SYS_DATA_MSB:0]  sys_data_i;
   output [`SYS_DATA_MSB:0] sys_data_o;
   input [7:1] 		    sys_cmd;
   input 		    sys_reset_n;
   input 		    sys_clk;
   output 		    sys_ref_ack; 		    
   input 		    sys_clk_fb;
   output 		    sys_data_valid;

   wire [`U_ADDR_MSB:0]     u_addr;
   wire [`U_DATA_MSB:0]     u_data_i;
   wire [`U_DATA_MSB:0]     u_data_o;
   wire [7:1] 		    u_cmd;

   ddr_ctlr I_ddr_ctlr (.ddr_ad(ddr_ad), .ddr_dm(ddr_dm), .ddr_ba(ddr_ba), 
	      .ddr_rasb(ddr_rasb), .ddr_casb(ddr_casb), .ddr_web(ddr_web), 
	      .ddr_clk(ddr_clk), .ddr_clkb(ddr_clkb), .ddr_dqs(ddr_dqs), 
	      .ddr_csb(ddr_csb), .ddr_cke(ddr_cke), .ddr_dq(ddr_dq),
	      .u_addr(u_addr), .u_cmd(u_cmd), .u_reset_n(u_reset_n), 
	      .u_data_o(u_data_o), .u_data_i(u_data_i),
	      .u_ref_ack(u_ref_ack), .u_data_valid(u_data_valid),
	      .u_clk(u_clk),  .u_clk_fb(u_clk_fb),
	      .fpga_clk(fpga_clk));
   
   user_int I_user_int (.sys_data_o(sys_data_o), .sys_ref_ack(sys_ref_ack), 
			.sys_data_valid(sys_data_valid), .u_addr(u_addr), 
			.u_data_i(u_data_i), .u_cmd(u_cmd), 
			.u_reset_n(u_reset_n), .u_clk(u_clk), 
			.u_clk_fb(u_clk_fb),
			.sys_addr(sys_addr), .sys_data_i(sys_data_i), 
			.sys_cmd(sys_cmd), .sys_reset_n(sys_reset_n), 
			.sys_clk(sys_clk), .sys_clk_fb(sys_clk_fb), 
			.u_data_o(u_data_o), 
			.u_ref_ack(u_ref_ack), .u_data_valid(u_data_valid), .fpga_clk(fpga_clk));
   
endmodule // top

