--*****************************************************************************
--
--    File Name:  top.vhd
--      Version:  0.9
--         Date:  Autgust 20, 1999
-- 
--  Description:  instantiate ddr_ctrl and a dummy user_int module.
--                In order to place and route the ddr_ctrl module, we need to
--                connect all user interface signals.  The user_int module simply
--                connects all user signals to IOs.  
-- 
--                The user should replace user_int module with their own interface.
-- 
-- 
--       Author:  Jennnifer Tran
--      Company:  Xilinx
--
--                Copyright (c) 1999 Xilinx, Inc.
--                All rights reserved
--
--    DDR SDRAM:  64M (1M x 16Bit x 4Bank)
-- 
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--*****************************************************************************/
library IEEE;
use IEEE.std_logic_1164.all;
library work;
use WORK.ddr_controller_comp.all;
-- synopsys translate_off
library unisim;
use unisim.vcomponents.all;
-- synopsys translate_on

entity top is
  port(
    --//Outputs
    ddr_ad	: out std_logic_vector(DDR_ADDR_MSB downto 0);
    ddr_dm	: out std_logic_vector(1 downto 0);
    ddr_ba	: out std_logic_vector(1 downto 0);
    ddr_rasb	: out std_logic;
    ddr_casb   : out std_logic;
    ddr_web   	: out std_logic;
    ddr_clk	: out std_logic;
    ddr_clkb	: out std_logic;
    ddr_dqs	: out std_logic_vector(1 downto 0);
    ddr_csb	: out std_logic;
    ddr_cke	: out std_logic;
    sys_data_o	: out std_logic_vector(SYS_DATA_MSB downto 0);
    sys_ref_ack: out std_logic;
    sys_data_valid : out std_logic;
    
    --//Inputs
    sys_addr	: in std_logic_vector(SYS_ADDR_MSB downto 0);
    sys_data_i	: in std_logic_vector(SYS_DATA_MSB downto 0);
    sys_cmd	: in std_logic_vector(7 downto 1);
    sys_reset_n : in std_logic;
    sys_clk	: in std_logic;
    sys_clk_fb	: in std_logic;
    
    --//Inouts
    ddr_dq	: inout std_logic_vector(DDR_DATA_MSB downto 0));
end top;

architecture rtl of top is
  signal u_addr       : std_logic_vector(U_ADDR_MSB downto 0);
  signal u_data_i     : std_logic_vector(U_DATA_MSB downto 0);
  signal u_data_o     : std_logic_vector(U_DATA_MSB downto 0);
  signal u_cmd	      : std_logic_vector(7 downto 1);
  signal u_reset_n    : std_logic;
  signal u_ref_ack    : std_logic;
  signal u_data_valid : std_logic;
  signal u_clk	      : std_logic;
  signal u_clk_fb     : std_logic;
  signal fpga_clk     : std_logic;
  component ddr_ctlr
    port(
      --Outputs
      ddr_ad	: out std_logic_vector(DDR_ADDR_MSB downto 0);
      ddr_dm	: out std_logic_vector(1 downto 0);
      ddr_ba	: out std_logic_vector(1 downto 0);
      ddr_rasb	: out std_logic;
      ddr_casb	: out std_logic;
      ddr_web	: out std_logic;
      ddr_clk	: out std_logic;
      ddr_clkb	: out std_logic;
      ddr_dqs	: out std_logic_vector(1 downto 0);
      ddr_csb	: out std_logic;
      ddr_cke	: out std_logic;
      u_data_o	: out std_logic_vector(U_DATA_MSB downto 0);
      u_ref_ack	: out std_logic;
      u_data_valid: out std_logic;
      fpga_clk	: out std_logic;
      --Inout
      ddr_dq	: inout std_logic_vector(DDR_DATA_MSB downto 0);
      --Inputs
      u_addr	: in std_logic_vector(U_ADDR_MSB downto 0);
      u_data_i	: in std_logic_vector(U_DATA_MSB downto 0);
      u_cmd	: in std_logic_vector(7 downto 1);
      u_reset_n	: in std_logic;
      u_clk	: in std_logic;
      u_clk_fb	: in std_logic);
  end component;
  component user_int
    port(
      --Outputs
      sys_data_o     : out std_logic_vector(SYS_DATA_MSB downto 0);
      sys_ref_ack    : out std_logic;
      sys_data_valid : out std_logic;
      u_addr	     : out std_logic_vector(U_ADDR_MSB downto 0);
      u_data_i	     : out std_logic_vector(U_DATA_MSB downto 0);
      u_cmd	     : out std_logic_vector(7 downto 1);
      u_reset_n	     : out std_logic;
      u_clk	     : out std_logic;
      u_clk_fb       : out std_logic;
      --Inputs
      sys_addr	     :in std_logic_vector(SYS_ADDR_MSB downto 0);
      sys_data_i     :in std_logic_vector(SYS_DATA_MSB downto 0);
      sys_cmd	     :in std_logic_vector(7 downto 1);
      sys_reset_n    :in std_logic;
      sys_clk	     :in std_logic;
      sys_clk_fb     :in std_logic;
      u_data_o	     :in std_logic_vector(U_DATA_MSB downto 0);
      u_ref_ack	     :in std_logic;
      u_data_valid   :in std_logic;
      fpga_clk	     :in std_logic);
  end component;

begin

  
  I_ddr_ctlr: ddr_ctlr
    port map(ddr_ad=>ddr_ad, ddr_dm=>ddr_dm,ddr_ba=>ddr_ba,
             ddr_rasb=>ddr_rasb, ddr_casb=>ddr_casb, ddr_web=>ddr_web,
             ddr_clk=>ddr_clk, ddr_clkb=>ddr_clkb, ddr_dqs=>ddr_dqs,
             ddr_csb=>ddr_csb, ddr_cke=>ddr_cke, ddr_dq=>ddr_dq,
             u_addr=>u_addr, u_cmd=>u_cmd, u_reset_n=>u_reset_n,
             u_data_o=>u_data_o, u_data_i=>u_data_i, u_ref_ack=>u_ref_ack,
             u_data_valid=>u_data_valid, u_clk=>u_clk, u_clk_fb=>u_clk_fb,
             fpga_clk=>fpga_clk);
  
  
  I_user_int: user_int
    port map(sys_data_o=>sys_data_o, sys_ref_ack=>sys_ref_ack,
             sys_data_valid=>sys_data_valid, u_addr=>u_addr,
             u_data_i=>u_data_i, u_cmd=>u_cmd, u_reset_n=>u_reset_n,
             u_clk=>u_clk, u_clk_fb=>u_clk_fb, sys_addr=>sys_addr,
             sys_data_i=>sys_data_i, sys_cmd=>sys_cmd,
             sys_reset_n=>sys_reset_n, sys_clk=>sys_clk,
             sys_clk_fb=>sys_clk_fb, u_data_o=>u_data_o,
             u_ref_ack=>u_ref_ack, u_data_valid=>u_data_valid,
             fpga_clk=>fpga_clk);

end rtl;
