--*****************************************************************************
--
--    File Name:  user_int.vhd
--      Version:  0.9
--         Date:  Autgust 20, 1999
-- 
--  Description:  dummy user_int module.
--                In order to place and route the ddr_ctrl module, we need to
--                connect all user interface signals.  The user_int module simply
--                connects all user signals to IOs.  
-- 
--                The user should replace user_int module with their own interface.
-- 
-- 
--       Author:  Jennnifer Tran
--      Company:  Xilinx
--
--                Copyright (c) 1999 Xilinx, Inc.
--                All rights reserved
--
--    DDR SDRAM:  64M (1M x 16Bit x 4Bank)
-- 
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--*****************************************************************************/
library IEEE;
use IEEE.std_logic_1164.all;
library work;
use WORK.ddr_controller_comp.all;
-- synopsys translate_off
library unisim;
use unisim.vcomponents.all;
-- synopsys translate_on

entity user_int is
  port(
    --Outputs
    sys_data_o	   : out std_logic_vector(SYS_DATA_MSB downto 0);
    sys_ref_ack    : out std_logic;
    sys_data_valid : out std_logic;
    u_addr	   : out std_logic_vector(U_ADDR_MSB downto 0);
    u_data_i	   : out std_logic_vector(U_DATA_MSB downto 0);
    u_cmd	   : out std_logic_vector(7 downto 1);
    u_reset_n	   : out std_logic;
    u_clk	   : out std_logic;
    u_clk_fb       : out std_logic;
    
    --Inputs
    sys_addr	   :in std_logic_vector(SYS_ADDR_MSB downto 0);
    sys_data_i	   :in std_logic_vector(SYS_DATA_MSB downto 0);
    sys_cmd	   :in std_logic_vector(7 downto 1);
    sys_reset_n    :in std_logic;
    sys_clk	   :in std_logic;
    sys_clk_fb	   :in std_logic;
    u_data_o	   :in std_logic_vector(U_DATA_MSB downto 0);
    u_ref_ack	   :in std_logic;
    u_data_valid   :in std_logic;
    fpga_clk	   :in std_logic);
end user_int;

architecture rtl of user_int is
  signal ddr_ad	           : std_logic_vector(DDR_ADDR_MSB downto 0);
  signal ddr_dq	           : std_logic_vector(DDR_DATA_MSB downto 0);
  signal ddr_dm	           : std_logic_vector(1 downto 0);
  signal ddr_ba	           : std_logic_vector(1 downto 0);
  signal ddr_dqs	   : std_logic_vector(1 downto 0);
  signal sys_addr_pad	   : std_logic_vector(SYS_ADDR_MSB downto 0);
  signal sys_ref_ack_pad   : std_logic;
  signal sys_data_i_pad	   : std_logic_vector(SYS_DATA_MSB downto 0);
  signal sys_data_i_pad_inv: std_logic_vector(SYS_DATA_MSB downto 0);
  signal sys_data_o_pad	   : std_logic_vector(SYS_DATA_MSB downto 0);
  signal sys_cmd_pad	   : std_logic_vector(7 downto 1);
  signal sys_reset_n_pad   : std_logic;
  signal sys_data_valid_pad: std_logic;
  signal data_o_sel        : std_logic_vector(7 downto 0);
  component IBUF_SSTL2_I
    port(I : in std_logic;
         O : out std_logic);
  end component;

  component OBUF_SSTL2_I
    port(I : in std_logic;
         O : out std_logic);
  end component;
 
begin
  data_o_sel <= "00000001";  
  sys_data_i_pad_inv <= not (sys_data_i_pad);
  
  process (fpga_clk)
  begin
    if (fpga_clk 'event and fpga_clk = '1') then
      u_addr <= sys_addr_pad;
      u_data_i(15 downto 0) <= sys_data_i_pad(15 downto 0) after 1 ns;
      u_data_i(31 downto 16) <= not sys_data_i_pad(15 downto 0) after 1 ns;
      u_data_i(47 downto 32) <= sys_data_i_pad(7 downto 0) & sys_data_i_pad(15 downto 8) after 1 ns;
      u_data_i(63 downto 48) <= not(sys_data_i_pad(7 downto 0) & sys_data_i_pad(15 downto 8)) after 1 ns;
      u_data_i(79 downto 64) <= not sys_data_i_pad(15 downto 0) after 1 ns;
      u_data_i(95 downto 80) <= sys_data_i_pad(15 downto 0) after 1 ns;
      u_data_i(111 downto 96) <= not(sys_data_i_pad(7 downto 0) & sys_data_i_pad(15 downto 8)) after 1 ns;
      u_data_i(127 downto 112) <= sys_data_i_pad(7 downto 0) & sys_data_i_pad(15 downto 8) after 1 ns;

      u_cmd <= sys_cmd_pad;
      u_reset_n <= sys_reset_n_pad;
      sys_data_valid_pad <= u_data_valid;
      g1: for i in 0 to 7 loop
        if data_o_sel(i) = '1' then
          sys_data_o_pad <= u_data_o( (16*i)+15 downto 16*i);
        end if;
      end loop;  -- i
    end if;
  end process;

  u_clk <= sys_clk;
  sys_ref_ack_pad <= u_ref_ack;
  u_clk_fb <= sys_clk_fb;
  
  g0: for i in 0 to SYS_ADDR_MSB generate
    u_addr: IBUF_SSTL2_I port map(I=>sys_addr(i), O=>sys_addr_pad(i));
  end generate; 
  
  g1: for i in 0 to SYS_DATA_MSB generate
    u_di: IBUF_SSTL2_I port map(I=>sys_data_i(i), O=>sys_data_i_pad(i));
    u_do: OBUF_SSTL2_I port map(O=>sys_data_o(i), I=>sys_data_o_pad(i));
  end generate;
  
  g2: for i in 1 to 7 generate
    u_cmd: IBUF_SSTL2_I port map(I=>sys_cmd(i), O=>sys_cmd_pad(i));
  end generate;
  
  u_reset0:      IBUF_SSTL2_I port map(I=>sys_reset_n, O=>sys_reset_n_pad);
  u_data_valid0: OBUF_SSTL2_I port map(O=>sys_data_valid, I=>sys_data_valid_pad);
  u_ref_ack0:    OBUF_SSTL2_I port map(O=>sys_ref_ack, I=>sys_ref_ack_pad);
	
end rtl;


